@@ -10,7 +10,7 @@ from torchvision.transforms import Compose, Normalize, Pad, RandomCrop, RandomHo
 
 import ignite.distributed as idist
 from ignite.contrib.engines import common
-from ignite.contrib.handlers import ProgressBar
+from ignite.handlers import ProgressBar
 from ignite.engine import Engine, Events, create_supervised_evaluator
 from ignite.metrics import Accuracy
 
@@ -220,4 +220,4 @@ if in_colab:
 
 
 # Full featured CIFAR10 example:
-# https://github.com/pytorch/ignite/tree/master/examples/contrib/cifar10
+# https://github.com/pytorch/ignite/tree/master/examples/cifar10
