@@ -4,9 +4,9 @@ from PIL import Image
 def load_image(filename, size=None, scale=None):
     img = Image.open(filename)
     if size is not None:
-        img = img.resize((size, size), Image.ANTIALIAS)
+        img = img.resize((size, size), Image.LANCZOS)
     elif scale is not None:
-        img = img.resize((int(img.size[0] / scale), int(img.size[1] / scale)), Image.ANTIALIAS)
+        img = img.resize((int(img.size[0] / scale), int(img.size[1] / scale)), Image.LANCZOS)
     return img
 
 
