@@ -9,9 +9,9 @@ import torch.nn as nn
 import torch.optim as optim
 import torch.utils.data as data
 
-from ignite.contrib.handlers import ProgressBar
 from ignite.engine import Engine, Events
-from ignite.handlers import ModelCheckpoint, Timer
+
+from ignite.handlers import ModelCheckpoint, ProgressBar, Timer
 from ignite.metrics import RunningAverage
 
 try:
@@ -85,7 +85,7 @@ class Generator(Net):
             nn.ReLU(inplace=True),
             # state size. (nf) x 32 x 32
             nn.ConvTranspose2d(in_channels=nf, out_channels=nc, kernel_size=4, stride=2, padding=1, bias=False),
-            nn.Tanh()
+            nn.Tanh(),
             # state size. (nc) x 64 x 64
         )
 
@@ -207,7 +207,6 @@ def main(
     alpha,
     output_dir,
 ):
-
     # seed
     check_manual_seed(seed)
 
@@ -243,7 +242,6 @@ def main(
 
     # The main function, processing a batch of examples
     def step(engine, batch):
-
         # unpack the batch. It comes from a dataset, so we have <images, labels> pairs. Discard labels.
         real, _ = batch
         real = real.to(device)
