@@ -15,24 +15,27 @@ except ImportError:
     from torch.optim.lr_scheduler import _LRScheduler as PyTorchLRScheduler
 
 import ignite.distributed as idist
-from ignite.contrib.handlers import (
+from ignite.engine import Engine, Events
+from ignite.handlers import (
+    Checkpoint,
     ClearMLLogger,
+    DiskSaver,
+    EarlyStopping,
     global_step_from_engine,
     MLflowLogger,
     NeptuneLogger,
     PolyaxonLogger,
     ProgressBar,
     TensorboardLogger,
+    TerminateOnNan,
     VisdomLogger,
     WandBLogger,
 )
-from ignite.contrib.handlers.base_logger import BaseLogger
-from ignite.contrib.metrics import GpuInfo
-from ignite.engine import Engine, Events
-from ignite.handlers import Checkpoint, DiskSaver, EarlyStopping, TerminateOnNan
+from ignite.handlers.base_logger import BaseLogger
 from ignite.handlers.checkpoint import BaseSaveHandler
 from ignite.handlers.param_scheduler import ParamScheduler
-from ignite.metrics import RunningAverage
+from ignite.metrics import GpuInfo, RunningAverage
+from ignite.metrics.metric import RunningBatchWise
 from ignite.utils import deprecated
 
 
@@ -48,7 +51,6 @@ def setup_common_training_handlers(
     with_pbars: bool = True,
     with_pbar_on_iters: bool = True,
     log_every_iters: int = 100,
-    device: Optional[Union[str, torch.device]] = None,
     stop_on_nan: bool = True,
     clear_cuda_cache: bool = True,
     save_handler: Optional[Union[Callable, BaseSaveHandler]] = None,
@@ -75,14 +77,14 @@ def setup_common_training_handlers(
             exclusive with ``save_handler``.
         lr_scheduler: learning rate scheduler
             as native torch LRScheduler or ignite's parameter scheduler.
-        with_gpu_stats: if True, :class:`~ignite.contrib.metrics.GpuInfo` is attached to the
+        with_gpu_stats: if True, :class:`~ignite.metrics.GpuInfo` is attached to the
             trainer. This requires `pynvml` package to be installed.
         output_names: list of names associated with `update_function` output dictionary.
         with_pbars: if True, two progress bars on epochs and optionally on iterations are attached.
             Default, True.
         with_pbar_on_iters: if True, a progress bar on iterations is attached to the trainer.
             Default, True.
-        log_every_iters: logging interval for :class:`~ignite.contrib.metrics.GpuInfo` and for
+        log_every_iters: logging interval for :class:`~ignite.metrics.GpuInfo` and for
             epoch-wise progress bar. Default, 100.
         stop_on_nan: if True, :class:`~ignite.handlers.terminate_on_nan.TerminateOnNan` handler is added to the trainer.
             Default, True.
@@ -92,10 +94,7 @@ def setup_common_training_handlers(
             class to use to store ``to_save``. See :class:`~ignite.handlers.checkpoint.Checkpoint` for more details.
             Argument is mutually exclusive with ``output_path``.
         kwargs: optional keyword args to be passed to construct :class:`~ignite.handlers.checkpoint.Checkpoint`.
-        device: deprecated argument, it will be removed in v0.5.0.
     """
-    if device is not None:
-        warnings.warn("Argument device is unused and deprecated. It will be removed in v0.5.0")
 
     if idist.get_world_size() > 1:
         _setup_common_distrib_training_handlers(
@@ -170,9 +169,7 @@ def _setup_common_training_handlers(
 
     if lr_scheduler is not None:
         if isinstance(lr_scheduler, PyTorchLRScheduler):
-            trainer.add_event_handler(
-                Events.ITERATION_COMPLETED, lambda engine: cast(PyTorchLRScheduler, lr_scheduler).step()
-            )
+            trainer.add_event_handler(Events.ITERATION_COMPLETED, lambda engine: lr_scheduler.step())
         else:
             trainer.add_event_handler(Events.ITERATION_STARTED, lr_scheduler)
 
@@ -180,7 +177,6 @@ def _setup_common_training_handlers(
         trainer.add_event_handler(Events.EPOCH_COMPLETED, empty_cuda_cache)
 
     if to_save is not None:
-
         if output_path is None and save_handler is None:
             raise ValueError(
                 "If to_save argument is provided then output_path or save_handler arguments should be also defined"
@@ -214,8 +210,8 @@ def _setup_common_training_handlers(
                 )
 
         for i, n in enumerate(output_names):
-            RunningAverage(output_transform=partial(output_transform, index=i, name=n), epoch_bound=False).attach(
-                trainer, n
+            RunningAverage(output_transform=partial(output_transform, index=i, name=n)).attach(
+                trainer, n, usage=RunningBatchWise()
             )
 
     if with_pbars:
@@ -246,7 +242,6 @@ def _setup_common_distrib_training_handlers(
     save_handler: Optional[Union[Callable, BaseSaveHandler]] = None,
     **kwargs: Any,
 ) -> None:
-
     _setup_common_training_handlers(
         trainer,
         to_save=to_save,
@@ -270,7 +265,7 @@ def _setup_common_distrib_training_handlers(
 
         @trainer.on(Events.EPOCH_STARTED)
         def distrib_set_epoch(engine: Engine) -> None:
-            cast(DistributedSampler, train_sampler).set_epoch(engine.state.epoch - 1)
+            train_sampler.set_epoch(engine.state.epoch - 1)
 
 
 def empty_cuda_cache(_: Engine) -> None:
@@ -368,7 +363,7 @@ def setup_tb_logging(
         kwargs: optional keyword args to be passed to construct the logger.
 
     Returns:
-        :class:`~ignite.contrib.handlers.tensorboard_logger.TensorboardLogger`
+        :class:`~ignite.handlers.tensorboard_logger.TensorboardLogger`
     """
     logger = TensorboardLogger(log_dir=output_path, **kwargs)
     _setup_logging(logger, trainer, optimizers, evaluators, log_every_iters)
@@ -399,7 +394,7 @@ def setup_visdom_logging(
         kwargs: optional keyword args to be passed to construct the logger.
 
     Returns:
-        :class:`~ignite.contrib.handlers.visdom_logger.VisdomLogger`
+        :class:`~ignite.handlers.visdom_logger.VisdomLogger`
     """
     logger = VisdomLogger(**kwargs)
     _setup_logging(logger, trainer, optimizers, evaluators, log_every_iters)
@@ -430,7 +425,7 @@ def setup_mlflow_logging(
         kwargs: optional keyword args to be passed to construct the logger.
 
     Returns:
-        :class:`~ignite.contrib.handlers.mlflow_logger.MLflowLogger`
+        :class:`~ignite.handlers.mlflow_logger.MLflowLogger`
     """
     logger = MLflowLogger(**kwargs)
     _setup_logging(logger, trainer, optimizers, evaluators, log_every_iters)
@@ -461,7 +456,7 @@ def setup_neptune_logging(
         kwargs: optional keyword args to be passed to construct the logger.
 
     Returns:
-        :class:`~ignite.contrib.handlers.neptune_logger.NeptuneLogger`
+        :class:`~ignite.handlers.neptune_logger.NeptuneLogger`
     """
     logger = NeptuneLogger(**kwargs)
     _setup_logging(logger, trainer, optimizers, evaluators, log_every_iters)
@@ -492,7 +487,7 @@ def setup_wandb_logging(
         kwargs: optional keyword args to be passed to construct the logger.
 
     Returns:
-        :class:`~ignite.contrib.handlers.wandb_logger.WandBLogger`
+        :class:`~ignite.handlers.wandb_logger.WandBLogger`
     """
     logger = WandBLogger(**kwargs)
     _setup_logging(logger, trainer, optimizers, evaluators, log_every_iters)
@@ -523,7 +518,7 @@ def setup_plx_logging(
         kwargs: optional keyword args to be passed to construct the logger.
 
     Returns:
-        :class:`~ignite.contrib.handlers.polyaxon_logger.PolyaxonLogger`
+        :class:`~ignite.handlers.polyaxon_logger.PolyaxonLogger`
     """
     logger = PolyaxonLogger(**kwargs)
     _setup_logging(logger, trainer, optimizers, evaluators, log_every_iters)
@@ -554,7 +549,7 @@ def setup_clearml_logging(
         kwargs: optional keyword args to be passed to construct the logger.
 
     Returns:
-        :class:`~ignite.contrib.handlers.clearml_logger.ClearMLLogger`
+        :class:`~ignite.handlers.clearml_logger.ClearMLLogger`
     """
     logger = ClearMLLogger(**kwargs)
     _setup_logging(logger, trainer, optimizers, evaluators, log_every_iters)
