@@ -15,7 +15,6 @@ has_native_dist_support = dist.is_available()
 
 
 if has_native_dist_support:
-
     NCCL = dist.Backend.NCCL
     GLOO = dist.Backend.GLOO
     MPI = dist.Backend.MPI
@@ -196,7 +195,6 @@ if has_native_dist_support:
             return local_rank
 
         def _identify_local_rank(self) -> None:
-
             if "SLURM_JOB_ID" in os.environ:
                 os.environ["LOCAL_RANK"] = os.environ["SLURM_LOCALID"]
 
@@ -216,7 +214,6 @@ if has_native_dist_support:
                 self._local_rank = self._compute_local_rank_via_hostname()
 
         def setup_env_vars(self, rank: Optional[int] = None, world_size: Optional[int] = None) -> None:
-
             self._env_backup = os.environ.copy()
 
             if "SLURM_JOB_ID" in os.environ:
@@ -426,6 +423,7 @@ if has_native_dist_support:
             if group is not None and not isinstance(group, dist.ProcessGroup):
                 raise ValueError("Argument group should be list of int or ProcessGroup")
             reduce_op = self._reduce_op_map[op]
+            # We do if/else here for compatibility with older pytorch versions
             if group is not None:
                 dist.all_reduce(tensor, reduce_op, group=group)
             else:
@@ -433,17 +431,50 @@ if has_native_dist_support:
             return tensor
 
         def _do_all_gather(self, tensor: torch.Tensor, group: Optional[Any] = None) -> torch.Tensor:
-            if group is not None and not isinstance(group, dist.ProcessGroup):
+            if group == dist.GroupMember.NON_GROUP_MEMBER:
+                return tensor
+
+            if group is None:
+                group_size = self.get_world_size()
+            elif isinstance(group, dist.ProcessGroup):
+                group_size = group.size()
+            else:
                 raise ValueError("Argument group should be list of int or ProcessGroup")
             if tensor.ndimension() == 0:
                 tensor = tensor.unsqueeze(0)
-            output = [torch.zeros_like(tensor) for _ in range(self.get_world_size())]
+            output = [torch.zeros_like(tensor) for _ in range(group_size)]
+            # We do if/else here for compatibility with older pytorch versions
             if group is not None:
                 dist.all_gather(output, tensor, group=group)
             else:
                 dist.all_gather(output, tensor)
             return torch.cat(output, dim=0)
 
+        def _do_all_gather_object(self, tensor: Any, group: Optional[Any] = None) -> List[Any]:
+            if Version(torch.__version__) < Version("1.7.0"):
+                raise RuntimeError(
+                    "Current torch version does not implement dist.all_gather_object. "
+                    "Required version should be >=1.7.0"
+                )
+
+            if group == dist.GroupMember.NON_GROUP_MEMBER:
+                return tensor
+
+            if group is None:
+                group_size = self.get_world_size()
+            elif isinstance(group, dist.ProcessGroup):
+                group_size = group.size()
+            else:
+                raise ValueError("Argument group should be list of int or ProcessGroup")
+            output = [None for _ in range(group_size)]
+            # We do if/else here for compatibility with older pytorch versions
+            if group is not None:
+                dist.all_gather_object(output, tensor, group=group)
+            else:
+                dist.all_gather_object(output, tensor)
+
+            return output
+
         def _do_new_group(self, ranks: List[int], **kwargs: Any) -> Any:
             return dist.new_group(ranks=ranks, **kwargs)
 
@@ -475,7 +506,6 @@ if has_native_dist_support:
         nodelist = nodelist.replace(" ", "")
 
         for node in re.findall(nodelist_match, nodelist):
-
             node_match = r"(.+)\[((,?[0-9]+-?,?-?){0,})\](.*)?"
 
             match = re.search(node_match, node)
