@@ -39,8 +39,14 @@ class Engine(Serializable):
 
         .. code-block:: python
 
-            def update_model(engine, batch):
-                inputs, targets = batch
+            model = ...
+            model = model.cuda()
+            optimized = ...
+            criterion = ...
+
+            def train_step(engine, batch):
+                model.train()
+                inputs, targets = batch[0].cuda(), batch[1].cuda()
                 optimizer.zero_grad()
                 outputs = model(inputs)
                 loss = criterion(outputs, targets)
@@ -150,7 +156,7 @@ class Engine(Serializable):
         _check_signature(process_function, "process_function", self, None)
 
         # generator provided by self._internal_run_as_gen
-        self._internal_run_generator: Optional[Generator] = None
+        self._internal_run_generator: Optional[Generator[Any, None, State]] = None
 
     def register_events(
         self, *event_names: Union[List[str], List[EventEnum]], event_to_attr: Optional[dict] = None
@@ -642,7 +648,7 @@ class Engine(Serializable):
         return self._state_dict_user_keys
 
     def state_dict(self) -> OrderedDict:
-        """Returns a dictionary containing engine's state: "seed", "epoch_length", "max_epochs" and "iteration" and
+        """Returns a dictionary containing engine's state: "epoch_length", "max_epochs" and "iteration" and
         other state values defined by `engine.state_dict_user_keys`
 
         .. code-block:: python
@@ -675,11 +681,11 @@ class Engine(Serializable):
     def load_state_dict(self, state_dict: Mapping) -> None:
         """Setups engine from `state_dict`.
 
-        State dictionary should contain keys: `iteration` or `epoch` and `max_epochs`, `epoch_length` and
-        `seed`. If `engine.state_dict_user_keys` contains keys, they should be also present in the state dictionary.
+        State dictionary should contain keys: `iteration` or `epoch`, `max_epochs` and `epoch_length`.
+        If `engine.state_dict_user_keys` contains keys, they should be also present in the state dictionary.
         Iteration and epoch values are 0-based: the first iteration or epoch is zero.
 
-        This method does not remove any custom attributs added by user.
+        This method does not remove any custom attributes added by user.
 
         Args:
             state_dict: a dict with parameters
@@ -773,13 +779,12 @@ class Engine(Serializable):
         data: Optional[Iterable] = None,
         max_epochs: Optional[int] = None,
         epoch_length: Optional[int] = None,
-        seed: Optional[int] = None,
     ) -> State:
         """Runs the ``process_function`` over the passed data.
 
         Engine has a state and the following logic is applied in this function:
 
-        - At the first call, new state is defined by `max_epochs`, `epoch_length`, `seed`, if provided.
+        - At the first call, new state is defined by `max_epochs`, `epoch_length`, if provided.
           A timer for total and per-epoch time is initialized when Events.STARTED is handled.
         - If state is already defined such that there are iterations to run until `max_epochs` and no input arguments
           provided, state is kept and used in the function.
@@ -797,8 +802,6 @@ class Engine(Serializable):
                 `len(data)`. If `data` is an iterator and `epoch_length` is not set, then it will be automatically
                 determined as the iteration on which data iterator raises `StopIteration`.
                 This argument should not change if run is resuming from a state.
-            seed: Deprecated argument. Please, use `torch.manual_seed` or :meth:`~ignite.utils.manual_seed`.
-
         Returns:
             State: output state.
 
@@ -826,12 +829,6 @@ class Engine(Serializable):
                 trainer.run(train_loader, max_epochs=2)
 
         """
-        if seed is not None:
-            warnings.warn(
-                "Argument seed is deprecated. It will be removed in 0.5.0. "
-                "Please, use torch.manual_seed or ignite.utils.manual_seed"
-            )
-
         if data is not None and not isinstance(data, Iterable):
             raise TypeError("Argument data should be iterable")
 
@@ -937,7 +934,7 @@ class Engine(Serializable):
             self._internal_run_generator = None
             return out.value
 
-    def _internal_run_as_gen(self) -> Generator:
+    def _internal_run_as_gen(self) -> Generator[Any, None, State]:
         self.should_terminate = self.should_terminate_single_epoch = self.should_interrupt = False
         self._init_timers(self.state)
         try:
@@ -1023,15 +1020,23 @@ class Engine(Serializable):
 
             while True:
                 self.state.batch = self.state.output = None
+
                 try:
                     # Avoid Events.GET_BATCH_STARTED triggered twice when data iter is restarted
                     if self.last_event_name != Events.DATALOADER_STOP_ITERATION:
-                        self._fire_event(Events.GET_BATCH_STARTED)
-                        yield from self._maybe_terminate_or_interrupt()
+                        # We should not trigger GET_BATCH_STARTED, GET_BATCH_COMPLETED, DATALOADER_STOP_ITERATION events
+                        # if no data was provided to engine.run(data=None, ...)
+                        if self.state.dataloader is not None:
+                            self._fire_event(Events.GET_BATCH_STARTED)
+                            yield from self._maybe_terminate_or_interrupt()
 
                     self.state.batch = next(self._dataloader_iter)
-                    self._fire_event(Events.GET_BATCH_COMPLETED)
-                    yield from self._maybe_terminate_or_interrupt()
+
+                    # We should not trigger GET_BATCH_STARTED, GET_BATCH_COMPLETED, DATALOADER_STOP_ITERATION events
+                    # if no data was provided to engine.run(data=None, ...)
+                    if self.state.dataloader is not None:
+                        self._fire_event(Events.GET_BATCH_COMPLETED)
+                        yield from self._maybe_terminate_or_interrupt()
 
                     iter_counter += 1
                     should_exit = False
@@ -1046,6 +1051,7 @@ class Engine(Serializable):
                     if should_exit:
                         if not self._is_done(self.state) and self.state.max_epochs is not None:
                             total_iters = self.state.epoch_length * self.state.max_epochs
+
                             warnings.warn(
                                 "Data iterator can not provide data anymore but required total number of "
                                 "iterations to run is not reached. "
@@ -1053,8 +1059,11 @@ class Engine(Serializable):
                             )
                         break
 
-                    self._fire_event(Events.DATALOADER_STOP_ITERATION)
-                    yield from self._maybe_terminate_or_interrupt()
+                    # We should not trigger GET_BATCH_STARTED, GET_BATCH_COMPLETED, DATALOADER_STOP_ITERATION events
+                    # if no data was provided to engine.run(data=None, ...)
+                    if self.state.dataloader is not None:
+                        self._fire_event(Events.DATALOADER_STOP_ITERATION)
+                        yield from self._maybe_terminate_or_interrupt()
 
                     self._setup_dataloader_iter()
                     should_exit = True
@@ -1173,12 +1182,18 @@ class Engine(Serializable):
                 try:
                     # Avoid Events.GET_BATCH_STARTED triggered twice when data iter is restarted
                     if self.last_event_name != Events.DATALOADER_STOP_ITERATION:
-                        self._fire_event(Events.GET_BATCH_STARTED)
-                        self._maybe_terminate_legacy()
+                        # We should not trigger GET_BATCH_STARTED, GET_BATCH_COMPLETED, DATALOADER_STOP_ITERATION events
+                        # if no data was provided to engine.run(data=None, ...)
+                        if self.state.dataloader is not None:
+                            self._fire_event(Events.GET_BATCH_STARTED)
+                            self._maybe_terminate_legacy()
 
                     self.state.batch = next(self._dataloader_iter)
-                    self._fire_event(Events.GET_BATCH_COMPLETED)
-                    self._maybe_terminate_legacy()
+                    # We should not trigger GET_BATCH_STARTED, GET_BATCH_COMPLETED, DATALOADER_STOP_ITERATION events
+                    # if no data was provided to engine.run(data=None, ...)
+                    if self.state.dataloader is not None:
+                        self._fire_event(Events.GET_BATCH_COMPLETED)
+                        self._maybe_terminate_legacy()
 
                     iter_counter += 1
                     should_exit = False
@@ -1193,6 +1208,7 @@ class Engine(Serializable):
                     if should_exit:
                         if not self._is_done(self.state) and self.state.max_epochs is not None:
                             total_iters = self.state.epoch_length * self.state.max_epochs
+
                             warnings.warn(
                                 "Data iterator can not provide data anymore but required total number of "
                                 "iterations to run is not reached. "
@@ -1200,8 +1216,11 @@ class Engine(Serializable):
                             )
                         break
 
-                    self._fire_event(Events.DATALOADER_STOP_ITERATION)
-                    self._maybe_terminate_legacy()
+                    # We should not trigger GET_BATCH_STARTED, GET_BATCH_COMPLETED, DATALOADER_STOP_ITERATION events
+                    # if no data was provided to engine.run(data=None, ...)
+                    if self.state.dataloader is not None:
+                        self._fire_event(Events.DATALOADER_STOP_ITERATION)
+                        self._maybe_terminate_legacy()
 
                     self._setup_dataloader_iter()
                     should_exit = True
