@@ -203,17 +203,6 @@ class CallableEventWithFilter:
         return EventsList() | self | other
 
 
-class CallableEvents(CallableEventWithFilter):
-    # For backward compatibility
-    def __init__(self, *args: Any, **kwargs: Any) -> None:
-        super(CallableEvents, self).__init__(*args, **kwargs)
-        warnings.warn(
-            "Class ignite.engine.events.CallableEvents is deprecated. It will be removed in 0.5.0. "
-            "Please, use ignite.engine.EventEnum instead",
-            DeprecationWarning,
-        )
-
-
 class EventEnum(CallableEventWithFilter, Enum):
     """Base class for all :class:`~ignite.engine.events.Events`. User defined custom events should also inherit
     this class.
