@@ -3,11 +3,10 @@ import numbers
 import os
 import stat
 import tempfile
-import warnings
 from abc import ABCMeta, abstractmethod
 from collections import OrderedDict
 from pathlib import Path
-from typing import Any, Callable, Dict, List, Mapping, NamedTuple, Optional, Tuple, Union
+from typing import Any, Callable, cast, Dict, List, Mapping, NamedTuple, Optional, Union
 
 import torch
 import torch.nn as nn
@@ -23,6 +22,7 @@ else:
 import ignite.distributed as idist
 from ignite.base import Serializable
 from ignite.engine import Engine, Events
+from ignite.utils import _tree_apply2, _tree_map
 
 __all__ = ["Checkpoint", "DiskSaver", "ModelCheckpoint", "BaseSaveHandler"]
 
@@ -82,7 +82,7 @@ class Checkpoint(Serializable):
             ``load_state_dict`` methods. If contains objects of type torch `DistributedDataParallel`_ or
             `DataParallel`_, their internal wrapped model is automatically saved (to avoid additional key ``module.`` in
             the state dictionary).
-        save_handler: String, method or callable class
+        save_handler: String, function or callable object.
             used to save engine and other provided objects. Function receives two objects: checkpoint as a dictionary
             and filename. If ``save_handler`` is callable class, it can
             inherit of :class:`~ignite.handlers.checkpoint.BaseSaveHandler` and optionally implement ``remove`` method
@@ -102,7 +102,6 @@ class Checkpoint(Serializable):
             Input of the function is ``(engine, event_name)``. Output of function should be an integer.
             Default is None, global_step based on attached engine. If provided, uses function output as global_step.
             To setup global step from another engine, please use :meth:`~ignite.handlers.global_step_from_engine`.
-        archived: Deprecated argument as models saved by ``torch.save`` are already compressed.
         filename_pattern: If ``filename_pattern`` is provided, this pattern will be used to render
             checkpoint filenames. If the pattern is not defined, the default pattern would be used. See Note for
             details.
@@ -278,7 +277,7 @@ class Checkpoint(Serializable):
     """
 
     Item = NamedTuple("Item", [("priority", int), ("filename", str)])
-    _state_dict_all_req_keys = ("saved",)
+    _state_dict_all_req_keys = ("_saved",)
 
     def __init__(
         self,
@@ -289,13 +288,11 @@ class Checkpoint(Serializable):
         score_name: Optional[str] = None,
         n_saved: Union[int, None] = 1,
         global_step_transform: Optional[Callable] = None,
-        archived: bool = False,
         filename_pattern: Optional[str] = None,
         include_self: bool = False,
         greater_or_equal: bool = False,
         save_on_rank: int = 0,
     ):
-
         if not isinstance(to_save, collections.Mapping):
             raise TypeError(f"Argument `to_save` should be a dictionary, but given {type(to_save)}")
 
@@ -322,8 +319,6 @@ class Checkpoint(Serializable):
 
         if global_step_transform is not None and not callable(global_step_transform):
             raise TypeError(f"global_step_transform should be a function, got {type(global_step_transform)} instead.")
-        if archived:
-            warnings.warn("Argument archived is deprecated and will be removed in 0.5.0")
 
         self.to_save = to_save
         self.filename_prefix = filename_prefix
@@ -405,7 +400,6 @@ class Checkpoint(Serializable):
             return new > self._saved[0].priority
 
     def __call__(self, engine: Engine) -> None:
-
         global_step = None
         if self.global_step_transform is not None:
             global_step = self.global_step_transform(engine, engine.last_event_name)
@@ -420,7 +414,6 @@ class Checkpoint(Serializable):
             priority = global_step
 
         if self._check_lt_n_saved() or self._compare_fn(priority):
-
             priority_str = f"{priority}" if isinstance(priority, numbers.Integral) else f"{priority:.4f}"
 
             checkpoint = self._setup_checkpoint()
@@ -473,18 +466,20 @@ class Checkpoint(Serializable):
             except TypeError:
                 self.save_handler(checkpoint, filename)
 
-    def _setup_checkpoint(self) -> Dict[str, Dict[Any, Any]]:
-        checkpoint = {}
+    def _setup_checkpoint(self) -> Dict[str, Any]:
         if self.to_save is not None:
-            for k, obj in self.to_save.items():
+
+            def func(obj: Any, **kwargs: Any) -> Dict:
                 if isinstance(obj, (nn.DataParallel, nn.parallel.DistributedDataParallel)):
                     obj = obj.module
                 elif HAVE_ZERO and isinstance(obj, ZeroRedundancyOptimizer):
                     obj.consolidate_state_dict(to=self.save_on_rank)
                     if self.save_on_rank != idist.get_rank():
-                        continue
-                checkpoint[k] = obj.state_dict()
-        return checkpoint
+                        return {}
+                return obj.state_dict()
+
+            return cast(Dict[str, Any], _tree_map(func, self.to_save))
+        return {}
 
     @staticmethod
     def setup_filename_pattern(
@@ -539,10 +534,12 @@ class Checkpoint(Serializable):
 
     @staticmethod
     def _check_objects(objs: Mapping, attr: str) -> None:
-        for k, obj in objs.items():
+        def func(obj: Any, **kwargs: Any) -> None:
             if not hasattr(obj, attr):
                 raise TypeError(f"Object {type(obj)} should have `{attr}` method")
 
+        _tree_map(func, objs)
+
     @staticmethod
     def load_objects(to_load: Mapping, checkpoint: Union[str, Mapping, Path], **kwargs: Any) -> None:
         """Helper method to apply ``load_state_dict`` on the objects from ``to_load`` using states from ``checkpoint``.
@@ -598,26 +595,22 @@ class Checkpoint(Serializable):
             torch.nn.parallel.DistributedDataParallel.html
         .. _DataParallel: https://pytorch.org/docs/stable/generated/torch.nn.DataParallel.html
         """
+        if not isinstance(checkpoint, (collections.Mapping, str, Path)):
+            raise TypeError(f"Argument checkpoint should be a string or a dictionary, but given {type(checkpoint)}")
+
+        Checkpoint._check_objects(to_load, "load_state_dict")
 
         if isinstance(checkpoint, (str, Path)):
             checkpoint_obj = torch.load(checkpoint)
         else:
             checkpoint_obj = checkpoint
 
-        Checkpoint._check_objects(to_load, "load_state_dict")
-        if not isinstance(checkpoint, (collections.Mapping, str, Path)):
-            raise TypeError(f"Argument checkpoint should be a string or a dictionary, but given {type(checkpoint)}")
-
-        if len(kwargs) > 1 or any(k for k in kwargs if k not in ["strict"]):
-            warnings.warn("kwargs contains keys other than strict and these will be ignored")
-
-        is_state_dict_strict = kwargs.get("strict", True)
-
         def _load_object(obj: Any, chkpt_obj: Any) -> None:
             if isinstance(obj, (nn.DataParallel, nn.parallel.DistributedDataParallel)):
                 obj = obj.module
+
             if isinstance(obj, torch.nn.Module):
-                obj.load_state_dict(chkpt_obj, strict=is_state_dict_strict)
+                obj.load_state_dict(chkpt_obj, **kwargs)
             else:
                 obj.load_state_dict(chkpt_obj)
 
@@ -628,11 +621,7 @@ class Checkpoint(Serializable):
                 _load_object(obj, checkpoint_obj)
                 return
 
-        # multiple objects to load
-        for k, obj in to_load.items():
-            if k not in checkpoint_obj:
-                raise ValueError(f"Object labeled by '{k}' from `to_load` is not found in the checkpoint")
-            _load_object(obj, checkpoint_obj[k])
+        _tree_apply2(_load_object, to_load, checkpoint_obj)
 
     def reload_objects(self, to_load: Mapping, load_kwargs: Optional[Dict] = None, **filename_components: Any) -> None:
         """Helper method to apply ``load_state_dict`` on the objects from ``to_load``. Filename components such as
@@ -676,10 +665,18 @@ class Checkpoint(Serializable):
             If ``to_load`` contains objects of type torch `DistributedDataParallel`_ or
             `DataParallel`_, method ``load_state_dict`` will applied to their internal wrapped model (``obj.module``).
 
+        Note:
+            This method works only when the ``save_handler`` is of types string,
+            :class:`~pathlib.Path` or :class:`~ignite.handlers.checkpoint.DiskSaver`.
+
         .. _DistributedDataParallel: https://pytorch.org/docs/stable/generated/
             torch.nn.parallel.DistributedDataParallel.html
         .. _DataParallel: https://pytorch.org/docs/stable/generated/torch.nn.DataParallel.html
         """
+        if not isinstance(self.save_handler, DiskSaver):
+            raise AttributeError(
+                f"Checkpoint's `save_handler` should be of type `DiskSaver`, given {type(self.save_handler)}"
+            )
 
         global_step = filename_components.get("global_step", None)
 
@@ -710,20 +707,21 @@ class Checkpoint(Serializable):
 
         Checkpoint.load_objects(to_load=to_load, checkpoint=path, **load_kwargs)
 
-    def state_dict(self) -> "OrderedDict[str, List[Tuple[int, str]]]":
+    def state_dict(self) -> OrderedDict:
         """Method returns state dict with saved items: list of ``(priority, filename)`` pairs.
         Can be used to save internal state of the class.
         """
-        return OrderedDict([("saved", [(p, f) for p, f in self._saved])])
+        # TODO: this method should use _state_dict_all_req_keys
+        return OrderedDict([("_saved", [(p, f) for p, f in self._saved])])
 
     def load_state_dict(self, state_dict: Mapping) -> None:
-        """Method replace internal state of the class with provided state dict data.
+        """Method replaces internal state of the class with provided state dict data.
 
         Args:
             state_dict: a dict with "saved" key and list of ``(priority, filename)`` pairs as values.
         """
         super().load_state_dict(state_dict)
-        self._saved = [Checkpoint.Item(p, f) for p, f in state_dict["saved"]]
+        self._saved = [Checkpoint.Item(p, f) for p, f in state_dict["_saved"]]
 
     @staticmethod
     def get_default_score_fn(metric_name: str, score_sign: float = 1.0) -> Callable:
@@ -882,11 +880,6 @@ class ModelCheckpoint(Checkpoint):
 
         Behaviour of this class has been changed since v0.3.0.
 
-        Argument ``save_as_state_dict`` is deprecated and should not be used. It is considered as True.
-
-        Argument ``save_interval`` is deprecated and should not be used. Please, use events filtering instead, e.g.
-        ``Events.ITERATION_STARTED(every=1000)``.
-
         There is no more internal counter that has been used to indicate the number of save actions. User could
         see its value `step_number` in the filename, e.g. `{filename_prefix}_{name}_{step_number}.pt`. Actually,
         `step_number` is replaced by current engine's epoch if `score_function` is specified and current iteration
@@ -915,7 +908,6 @@ class ModelCheckpoint(Checkpoint):
             Input of the function is `(engine, event_name)`. Output of function should be an integer.
             Default is None, global_step based on attached engine. If provided, uses function output as global_step.
             To setup global step from another engine, please use :meth:`~ignite.handlers.global_step_from_engine`.
-        archived: Deprecated argument as models saved by `torch.save` are already compressed.
         filename_pattern: If ``filename_pattern`` is provided, this pattern will be used to render
             checkpoint filenames. If the pattern is not defined, the default pattern would be used.
             See :class:`~ignite.handlers.checkpoint.Checkpoint` for details.
@@ -962,39 +954,19 @@ class ModelCheckpoint(Checkpoint):
         self,
         dirname: Union[str, Path],
         filename_prefix: str = "",
-        save_interval: Optional[int] = None,
         score_function: Optional[Callable] = None,
         score_name: Optional[str] = None,
         n_saved: Union[int, None] = 1,
         atomic: bool = True,
         require_empty: bool = True,
         create_dir: bool = True,
-        save_as_state_dict: bool = True,
         global_step_transform: Optional[Callable] = None,
-        archived: bool = False,
         filename_pattern: Optional[str] = None,
         include_self: bool = False,
         greater_or_equal: bool = False,
         save_on_rank: int = 0,
         **kwargs: Any,
     ):
-        if not save_as_state_dict:
-            raise ValueError(
-                "Argument save_as_state_dict is deprecated and should be True."
-                "This argument will be removed in 0.5.0."
-            )
-        if save_interval is not None:
-            msg = (
-                "Argument save_interval is deprecated and should be None. This argument will be removed in 0.5.0."
-                "Please, use events filtering instead, e.g. Events.ITERATION_STARTED(every=1000)"
-            )
-            if save_interval == 1:
-                # Do not break for old version who used `save_interval=1`
-                warnings.warn(msg)
-            else:
-                # No choice
-                raise ValueError(msg)
-
         disk_saver = DiskSaver(
             dirname,
             atomic=atomic,
@@ -1013,7 +985,6 @@ class ModelCheckpoint(Checkpoint):
             n_saved=n_saved,
             global_step_transform=global_step_transform,
             filename_pattern=filename_pattern,
-            archived=archived,
             include_self=include_self,
             greater_or_equal=greater_or_equal,
             save_on_rank=save_on_rank,
@@ -1030,7 +1001,6 @@ class ModelCheckpoint(Checkpoint):
         return self.save_handler.dirname / self._saved[-1].filename
 
     def __call__(self, engine: Engine, to_save: Mapping):  # type: ignore
-
         if len(to_save) == 0:
             raise RuntimeError("No objects to checkpoint found.")
 
