@@ -94,7 +94,6 @@ class FastaiLRFinder:
         smooth_f: float,
         diverge_th: float,
     ) -> None:
-
         self._history = {"lr": [], "loss": []}
         self._best_loss = None
         self._diverge_flag = False
@@ -161,13 +160,19 @@ class FastaiLRFinder:
                     raise ValueError(
                         "if output of the engine is torch.Tensor, then "
                         "it must be 0d torch.Tensor or 1d torch.Tensor with 1 element, "
-                        f"but got torch.Tensor of shape {loss.shape}"
+                        f"but got torch.Tensor of shape {loss.shape}."
                     )
             else:
                 raise TypeError(
                     "output of the engine should be of type float or 0d torch.Tensor "
                     "or 1d torch.Tensor with 1 element, "
                     f"but got output of type {type(loss).__name__}"
+                    "You may wish to use the output_transform kwarg with the attach method e.g.\n"
+                    """
+                    lr_finder = FastaiLRFinder()
+                    with lr_finder.attach(trainer, output_transform=lambda x:x["train_loss"]) as trainer_with_lr_finder:
+                        trainer_with_lr_finder.run(dataloader_train)
+                    """
                 )
         loss = idist.all_reduce(loss)
         lr = self._lr_schedule.get_param()
@@ -536,7 +541,7 @@ class _ExponentialLR(_LRScheduler):
         # override base_lrs
         self.base_lrs = start_lrs
 
-    def get_lr(self) -> List[float]:  # type: ignore[override]
+    def get_lr(self) -> List[float]:
         curr_iter = self.last_epoch + 1
         r = curr_iter / self.num_iter
         return [base_lr * (end_lr / base_lr) ** r for end_lr, base_lr in zip(self.end_lrs, self.base_lrs)]
