@@ -3,6 +3,7 @@ import os
 import pytest
 import torch
 import torch.distributed as dist
+from packaging.version import Version
 
 import ignite.distributed as idist
 from ignite.distributed.utils import has_native_dist_support
@@ -37,7 +38,6 @@ def _test_native_distrib_single_node_launch_tool(backend, device, local_rank, wo
 @pytest.mark.skipif(not has_native_dist_support, reason="Skip if no native dist support")
 @pytest.mark.parametrize("init_method", [None, "tcp://0.0.0.0:22334", "FILE"])
 def test_native_distrib_single_node_launch_tool_gloo(init_method, get_fixed_dirname, local_rank, world_size):
-
     from datetime import timedelta
 
     timeout = timedelta(seconds=20)
@@ -56,7 +56,6 @@ def test_native_distrib_single_node_launch_tool_gloo(init_method, get_fixed_dirn
 @pytest.mark.skipif(torch.cuda.device_count() < 1, reason="Skip if no GPU")
 @pytest.mark.parametrize("init_method", [None, "tcp://0.0.0.0:22334", "FILE"])
 def test_native_distrib_single_node_launch_tool_nccl(init_method, get_fixed_dirname, local_rank, world_size):
-
     if init_method == "FILE":
         init_method = f"file://{get_fixed_dirname('native_distrib_single_node_launch_tool_nccl')}/shared"
 
@@ -81,7 +80,6 @@ def _test_native_distrib_single_node_spawn(init_method, backend, device, **kwarg
 @pytest.mark.skipif("WORLD_SIZE" in os.environ, reason="Skip if launched as multiproc")
 @pytest.mark.parametrize("init_method", [None, "tcp://0.0.0.0:22334", "FILE"])
 def test_native_distrib_single_node_spawn_gloo(init_method, dirname):
-
     from datetime import timedelta
 
     timeout = timedelta(seconds=20)
@@ -190,7 +188,6 @@ def _test_idist_methods_in_native_context_set_local_rank(backend, device, local_
 @pytest.mark.distributed
 @pytest.mark.skipif(not has_native_dist_support, reason="Skip if no native dist support")
 def test_idist_methods_in_native_gloo_context_set_local_rank(distributed_context_single_node_gloo):
-
     local_rank = distributed_context_single_node_gloo["local_rank"]
     device = idist.device()
     _test_idist_methods_in_native_context_set_local_rank("gloo", device, local_rank)
@@ -209,7 +206,6 @@ def test_idist_methods_in_native_nccl_context_set_local_rank(distributed_context
 @pytest.mark.skipif(not has_native_dist_support, reason="Skip if no native dist support")
 @pytest.mark.skipif(torch.cuda.device_count() < 1, reason="Skip if no GPU")
 def test_idist__model_methods_nccl(distributed_context_single_node_nccl):
-
     device = idist.device()
     _test_distrib__get_max_length(device)
 
@@ -217,7 +213,6 @@ def test_idist__model_methods_nccl(distributed_context_single_node_nccl):
 @pytest.mark.distributed
 @pytest.mark.skipif(not has_native_dist_support, reason="Skip if no native dist support")
 def test_idist__model_methods_gloo(distributed_context_single_node_gloo):
-
     device = idist.device()
     _test_distrib__get_max_length(device)
 
@@ -226,7 +221,6 @@ def test_idist__model_methods_gloo(distributed_context_single_node_gloo):
 @pytest.mark.skipif(not has_native_dist_support, reason="Skip if no native dist support")
 @pytest.mark.skipif(torch.cuda.device_count() < 1, reason="Skip if no GPU")
 def test_idist_all_reduce_nccl(distributed_context_single_node_nccl):
-
     device = idist.device()
     _test_distrib_all_reduce(device)
     _test_distrib_all_reduce_group(device)
@@ -235,7 +229,6 @@ def test_idist_all_reduce_nccl(distributed_context_single_node_nccl):
 @pytest.mark.distributed
 @pytest.mark.skipif(not has_native_dist_support, reason="Skip if no native dist support")
 def test_idist_all_reduce_gloo(distributed_context_single_node_gloo):
-
     device = idist.device()
     _test_distrib_all_reduce(device)
     _test_distrib_all_reduce_group(device)
@@ -244,8 +237,8 @@ def test_idist_all_reduce_gloo(distributed_context_single_node_gloo):
 @pytest.mark.distributed
 @pytest.mark.skipif(not has_native_dist_support, reason="Skip if no native dist support")
 @pytest.mark.skipif(torch.cuda.device_count() < 1, reason="Skip if no GPU")
+@pytest.mark.skipif(Version(torch.__version__) < Version("1.7.0"), reason="dist.all_gather_object is not implemented")
 def test_idist_all_gather_nccl(distributed_context_single_node_nccl):
-
     device = idist.device()
     _test_distrib_all_gather(device)
     _test_distrib_all_gather_group(device)
@@ -253,8 +246,8 @@ def test_idist_all_gather_nccl(distributed_context_single_node_nccl):
 
 @pytest.mark.distributed
 @pytest.mark.skipif(not has_native_dist_support, reason="Skip if no native dist support")
+@pytest.mark.skipif(Version(torch.__version__) < Version("1.7.0"), reason="dist.all_gather_object is not implemented")
 def test_idist_all_gather_gloo(distributed_context_single_node_gloo):
-
     device = idist.device()
     _test_distrib_all_gather(device)
     _test_distrib_all_gather_group(device)
@@ -264,7 +257,6 @@ def test_idist_all_gather_gloo(distributed_context_single_node_gloo):
 @pytest.mark.skipif(not has_native_dist_support, reason="Skip if no native dist support")
 @pytest.mark.skipif(torch.cuda.device_count() < 1, reason="Skip if no GPU")
 def test_idist_broadcast_nccl(distributed_context_single_node_nccl):
-
     device = idist.device()
     _test_distrib_broadcast(device)
 
@@ -272,7 +264,6 @@ def test_idist_broadcast_nccl(distributed_context_single_node_nccl):
 @pytest.mark.distributed
 @pytest.mark.skipif(not has_native_dist_support, reason="Skip if no native dist support")
 def test_idist_broadcast_gloo(distributed_context_single_node_gloo):
-
     device = idist.device()
     _test_distrib_broadcast(device)
 
@@ -281,7 +272,6 @@ def test_idist_broadcast_gloo(distributed_context_single_node_gloo):
 @pytest.mark.skipif(not has_native_dist_support, reason="Skip if no native dist support")
 @pytest.mark.skipif(torch.cuda.device_count() < 1, reason="Skip if no GPU")
 def test_idist_barrier_nccl(distributed_context_single_node_nccl):
-
     device = idist.device()
     _test_distrib_barrier(device)
 
@@ -289,7 +279,6 @@ def test_idist_barrier_nccl(distributed_context_single_node_nccl):
 @pytest.mark.distributed
 @pytest.mark.skipif(not has_native_dist_support, reason="Skip if no native dist support")
 def test_idist_barrier_gloo(distributed_context_single_node_gloo):
-
     device = idist.device()
     _test_distrib_barrier(device)
 
@@ -356,7 +345,6 @@ def test_idist_methods_overhead_nccl(distributed_context_single_node_nccl):
 @pytest.mark.distributed
 @pytest.mark.skipif(not has_native_dist_support, reason="Skip if no native dist support")
 def test_idist_one_rank_only_gloo(distributed_context_single_node_gloo):
-
     device = idist.device()
     _test_distrib_one_rank_only(device=device)
     _test_distrib_one_rank_only_with_engine(device=device)
@@ -366,7 +354,48 @@ def test_idist_one_rank_only_gloo(distributed_context_single_node_gloo):
 @pytest.mark.skipif(not has_native_dist_support, reason="Skip if no native dist support")
 @pytest.mark.skipif(torch.cuda.device_count() < 1, reason="Skip if no GPU")
 def test_idist_one_rank_only_nccl(local_rank, distributed_context_single_node_nccl):
-
     device = idist.device()
     _test_distrib_one_rank_only(device=device)
     _test_distrib_one_rank_only_with_engine(device=device)
+
+
+@pytest.mark.distributed
+@pytest.mark.parametrize("rank", range(int(os.environ.get("WORLD_SIZE", 1))))
+@pytest.mark.parametrize("local", [True, False])
+def test_one_rank_first(distributed, get_rank_zero_dirname, rank, local):
+    def get_ds(file_path):
+        rank = idist.get_local_rank() if local else idist.get_rank()
+        if not file_path.exists():
+            with open(file_path, "w") as f:
+                f.write("readed")
+            return f"{rank} not readed"
+        else:
+            return f"{rank} readed"
+
+    folder = get_rank_zero_dirname()
+    file_path = folder / "res.txt"
+
+    with idist.one_rank_first(rank, local=local):
+        x = get_ds(file_path)
+
+    output = idist.all_gather(x)
+
+    if local:
+        expected = [
+            f"{x} not readed" if x == rank else f"{x} readed" for x in range(idist.get_nproc_per_node())
+        ] * idist.get_nnodes()
+    else:
+        expected = [f"{x} not readed" if x == rank else f"{x} readed" for x in range(idist.get_world_size())]
+
+    print("expected:", expected, idist.get_nnodes())
+    assert set(expected) == set(output)
+
+
+@pytest.mark.distributed
+def test_one_rank_first_asserts():
+    rank = 100
+    with pytest.raises(
+        ValueError, match=f"rank should be between 0 and {idist.get_world_size() - 1}, but given {rank}"
+    ):
+        with idist.one_rank_first(rank):
+            pass
