@@ -1,26 +1,34 @@
 import os
 import random
 import sys
+from collections.abc import Mapping
 from unittest.mock import patch
 
 import numpy as np
 import pytest
 import torch
 import torch.nn as nn
-from torch.utils.data import DataLoader
+from torch.optim import SGD
+from torch.utils.data import BatchSampler, DataLoader, RandomSampler
 
 import ignite.distributed as idist
 from ignite.engine import Events
 from ignite.engine.deterministic import (
+    _set_rng_states,
     DeterministicEngine,
-    ReproducibleBatchSampler,
     keep_random_state,
+    ReproducibleBatchSampler,
     update_dataloader,
 )
 from ignite.utils import manual_seed
 from tests.ignite.engine import BatchChecker, setup_sampler
 
 
+def test_dengine_setup_seed_div_by_zero():
+    with pytest.raises(ValueError, match=r"iter_counter should be positive value"):
+        DeterministicEngine(lambda e, b: None)._setup_seed(iter_counter=0)
+
+
 def test_update_dataloader():
     def _test(sampler_type=None):
         num_epochs = 3
@@ -87,9 +95,6 @@ def test_reproducible_batch_sampler_wrong_input():
 
 
 def test_reproducible_batch_sampler():
-    import torch
-    from torch.utils.data import DataLoader
-
     data = list(range(100))
     dataloader = DataLoader(data, batch_size=12, num_workers=0, shuffle=True, drop_last=True)
 
@@ -115,12 +120,10 @@ def test_reproducible_batch_sampler():
         resumed_seen_batches = []
         for b in dataloader_:
             resumed_seen_batches.append(b)
-
         assert all([(b1 == b2).all() for b1, b2 in zip(seen_batches[resume_epoch], resumed_seen_batches)])
 
 
 def _test_keep_random_state(with_numpy):
-
     manual_seed(54)
     true_values = []
     for _ in range(5):
@@ -170,7 +173,6 @@ def test_keep_random_state_without_numpy():
 
 def test_strict_resume_from_iter():
     def _test(epoch_length=None):
-
         max_epochs = 5
         num_iters = 21
         torch.manual_seed(0)
@@ -239,7 +241,6 @@ def test_strict_resume_from_epoch():
 
 def _test_resume_random_dataloader_from_epoch(device, _setup_sampler, sampler_type=None):
     def _test(epoch_length=None):
-
         max_epochs = 5
         total_batch_size = 4
         num_iters = 21
@@ -250,7 +251,6 @@ def _test_resume_random_dataloader_from_epoch(device, _setup_sampler, sampler_ty
             epoch_length = num_iters
 
         for resume_epoch in range(1, max_epochs, 2):
-
             for num_workers in [0, 2]:
                 sampler, batch_size = _setup_sampler(sampler_type, num_iters, total_batch_size)
 
@@ -258,7 +258,7 @@ def _test_resume_random_dataloader_from_epoch(device, _setup_sampler, sampler_ty
                     data,
                     batch_size=batch_size,
                     num_workers=num_workers,
-                    pin_memory="cuda" in device,
+                    pin_memory="cuda" in torch.device(device).type,
                     sampler=sampler,
                     drop_last=True,
                     shuffle=sampler is None,
@@ -279,9 +279,7 @@ def _test_resume_random_dataloader_from_epoch(device, _setup_sampler, sampler_ty
                         sampler.set_epoch(engine.state.epoch - 1)
 
                 torch.manual_seed(87)
-                engine.run(
-                    orig_dataloader, max_epochs=max_epochs, epoch_length=epoch_length,
-                )
+                engine.run(orig_dataloader, max_epochs=max_epochs, epoch_length=epoch_length)
 
                 batch_checker = BatchChecker(seen_batchs, init_counter=resume_epoch * epoch_length)
 
@@ -290,7 +288,7 @@ def _test_resume_random_dataloader_from_epoch(device, _setup_sampler, sampler_ty
                     data,
                     batch_size=batch_size,
                     num_workers=num_workers,
-                    pin_memory="cuda" in device,
+                    pin_memory="cuda" in torch.device(device).type,
                     sampler=sampler,
                     drop_last=True,
                     shuffle=sampler is None,
@@ -358,15 +356,13 @@ def _test_resume_random_dataloader_from_iter(device, _setup_sampler, sampler_typ
             epoch_length = num_iters
 
         for resume_iteration in range(2, min(num_iters * max_epochs, epoch_length * max_epochs), 13):
-
             for num_workers in [0, 2]:
-
                 sampler, batch_size = _setup_sampler(sampler_type, num_iters, total_batch_size)
                 orig_dataloader = DataLoader(
                     data,
                     batch_size=batch_size,
                     num_workers=num_workers,
-                    pin_memory="cuda" in device,
+                    pin_memory="cuda" in torch.device(device).type,
                     sampler=sampler,
                     drop_last=True,
                     shuffle=sampler is None,
@@ -386,9 +382,7 @@ def _test_resume_random_dataloader_from_iter(device, _setup_sampler, sampler_typ
                         sampler.set_epoch(engine.state.epoch)
 
                 torch.manual_seed(12)
-                engine.run(
-                    orig_dataloader, max_epochs=max_epochs, epoch_length=epoch_length,
-                )
+                engine.run(orig_dataloader, max_epochs=max_epochs, epoch_length=epoch_length)
 
                 batch_checker = BatchChecker(seen_batchs, init_counter=resume_iteration)
 
@@ -397,7 +391,7 @@ def _test_resume_random_dataloader_from_iter(device, _setup_sampler, sampler_typ
                     data,
                     batch_size=batch_size,
                     num_workers=num_workers,
-                    pin_memory="cuda" in device,
+                    pin_memory="cuda" in torch.device(device).type,
                     sampler=sampler,
                     drop_last=True,
                     shuffle=sampler is None,
@@ -467,9 +461,7 @@ def _test_resume_random_data_iterator_from_epoch(device):
 
             engine = DeterministicEngine(update_fn)
             torch.manual_seed(121)
-            engine.run(
-                infinite_data_iterator(), max_epochs=max_epochs, epoch_length=epoch_length,
-            )
+            engine.run(infinite_data_iterator(), max_epochs=max_epochs, epoch_length=epoch_length)
 
             batch_checker = BatchChecker(seen_batchs, init_counter=resume_epoch * epoch_length)
 
@@ -514,7 +506,6 @@ def _test_resume_random_data_iterator_from_iter(device):
             epoch_length = num_iters
 
         for resume_iteration in range(1, min(num_iters * max_epochs, epoch_length * max_epochs), 7):
-
             seen_batchs = []
 
             def update_fn(_, batch):
@@ -523,9 +514,7 @@ def _test_resume_random_data_iterator_from_iter(device):
             engine = DeterministicEngine(update_fn)
 
             torch.manual_seed(24)
-            engine.run(
-                infinite_data_iterator(), max_epochs=max_epochs, epoch_length=epoch_length,
-            )
+            engine.run(infinite_data_iterator(), max_epochs=max_epochs, epoch_length=epoch_length)
 
             batch_checker = BatchChecker(seen_batchs, init_counter=resume_iteration)
 
@@ -559,25 +548,26 @@ def test_resume_random_data_iterator_from_iter():
 @pytest.mark.distributed
 @pytest.mark.skipif(not idist.has_native_dist_support, reason="Skip if no native dist support")
 @pytest.mark.skipif(torch.cuda.device_count() < 1, reason="Skip if no GPU")
-def test_distrib_gpu(distributed_context_single_node_nccl):
-    device = f"cuda:{distributed_context_single_node_nccl['local_rank']}"
+def test_distrib_nccl_gpu(distributed_context_single_node_nccl):
+    device = idist.device()
     _test_resume_random_dataloader_from_iter(device, setup_sampler, sampler_type="distributed")
     _test_resume_random_dataloader_from_epoch(device, setup_sampler, sampler_type="distributed")
 
 
 @pytest.mark.distributed
 @pytest.mark.skipif(not idist.has_native_dist_support, reason="Skip if no native dist support")
-def test_distrib_cpu(distributed_context_single_node_gloo):
-    device = "cpu"
+def test_distrib_gloo_cpu_or_gpu(distributed_context_single_node_gloo):
+    device = idist.device()
     _test_resume_random_dataloader_from_iter(device, setup_sampler, sampler_type="distributed")
     _test_resume_random_dataloader_from_epoch(device, setup_sampler, sampler_type="distributed")
 
 
+@pytest.mark.xfail
 @pytest.mark.multinode_distributed
 @pytest.mark.skipif(not idist.has_native_dist_support, reason="Skip if no native dist support")
 @pytest.mark.skipif("MULTINODE_DISTRIB" not in os.environ, reason="Skip if not multi-node distributed")
-def test_multinode_distrib_cpu(distributed_context_multi_node_gloo):
-    device = "cpu"
+def test_multinode_distrib_gloo_cpu_or_gpu(distributed_context_multi_node_gloo):
+    device = idist.device()
     _test_resume_random_dataloader_from_iter(device, setup_sampler, sampler_type="distributed")
     _test_resume_random_dataloader_from_epoch(device, setup_sampler, sampler_type="distributed")
 
@@ -585,19 +575,19 @@ def test_multinode_distrib_cpu(distributed_context_multi_node_gloo):
 @pytest.mark.multinode_distributed
 @pytest.mark.skipif(not idist.has_native_dist_support, reason="Skip if no native dist support")
 @pytest.mark.skipif("GPU_MULTINODE_DISTRIB" not in os.environ, reason="Skip if not multi-node distributed")
-def test_multinode_distrib_gpu(distributed_context_multi_node_nccl):
-    device = f"cuda:{distributed_context_multi_node_nccl['local_rank']}"
+def test_multinode_distrib_nccl_gpu(distributed_context_multi_node_nccl):
+    device = idist.device()
     _test_resume_random_dataloader_from_iter(device, setup_sampler, sampler_type="distributed")
     _test_resume_random_dataloader_from_epoch(device, setup_sampler, sampler_type="distributed")
 
 
 def test_concepts_snippet_resume():
+    # Commented imports required in the snippet
+    # import torch
+    # from torch.utils.data import DataLoader
 
-    import torch
-    from torch.utils.data import DataLoader
-
-    from ignite.engine import DeterministicEngine, Events
-    from ignite.utils import manual_seed
+    # from ignite.engine import DeterministicEngine
+    # from ignite.utils import manual_seed
 
     seen_batches = []
     manual_seed(seed=15)
@@ -656,11 +646,7 @@ def test_concepts_snippet_warning():
 def _test_gradients_on_resume(
     dirname, device, with_dropout=True, with_dataaugs=True, data_size=24, batch_size=4, save_iter=None, save_epoch=None
 ):
-
-    debug = True
-
-    from torch.optim import SGD
-    from torch.utils.data import DataLoader
+    debug = False
 
     def random_train_data_loader(size):
         d = AugmentedData(torch.rand(size, 3, 32, 32), enabled=with_dataaugs)
@@ -717,7 +703,7 @@ def _test_gradients_on_resume(
         def save_chkpt(_):
             if debug:
                 print(trainer.state.iteration, "save_chkpt")
-            fp = os.path.join(dirname, "test.pt")
+            fp = dirname / "test.pt"
             from ignite.engine.deterministic import _repr_rng_state
 
             tsd = trainer.state_dict()
@@ -814,7 +800,6 @@ def test_gradients_on_resume_on_cuda(dirname):
 
 def test_engine_with_dataloader_no_auto_batching():
     # tests https://github.com/pytorch/ignite/issues/941
-    from torch.utils.data import BatchSampler, DataLoader, RandomSampler
 
     data = torch.rand(64, 4, 10)
     data_loader = DataLoader(
@@ -879,3 +864,29 @@ def test_dataloader_no_dataset_kind():
     dataloader = OldDataLoader(dataloader)
 
     engine.run(dataloader)
+
+
+@pytest.mark.skipif(not torch.cuda.is_available(), reason="Skip if no GPU")
+def test__set_rng_states_cuda():
+    # Checks https://github.com/pytorch/ignite/issues/2076
+
+    rng_states = [random.getstate(), torch.get_rng_state().cuda(), np.random.get_state()]
+    _set_rng_states(rng_states)
+    assert rng_states[1].device.type == "cpu"
+
+
+def test_engine_no_data_asserts():
+    trainer = DeterministicEngine(lambda e, b: None)
+
+    with pytest.raises(ValueError, match=r"Deterministic engine does not support the option of data=None"):
+        trainer.run(max_epochs=10, epoch_length=10)
+
+
+def test_state_dict():
+    engine = DeterministicEngine(lambda e, b: 1)
+    sd = engine.state_dict()
+    assert isinstance(sd, Mapping) and len(sd) == 4
+    assert "iteration" in sd and sd["iteration"] == 0
+    assert "max_epochs" in sd and sd["max_epochs"] is None
+    assert "epoch_length" in sd and sd["epoch_length"] is None
+    assert "rng_states" in sd and sd["rng_states"] is not None
