@@ -45,13 +45,15 @@ class DummyPretrainedModel(nn.Module):
 
 
 def test_checkpoint_wrong_input():
-
     with pytest.raises(TypeError, match=r"Argument `to_save` should be a dictionary"):
         Checkpoint(12, lambda x: x, "prefix")
 
     with pytest.raises(TypeError, match=r"Argument `to_save` should be a dictionary"):
         Checkpoint([12], lambda x: x, "prefix")
 
+    with pytest.raises(TypeError, match=r"should have `state_dict`"):
+        Checkpoint({"model": {"abc": 12}}, lambda x: x, "prefix")
+
     to_save = {"model": model}
 
     with pytest.raises(
@@ -63,28 +65,31 @@ def test_checkpoint_wrong_input():
     with pytest.raises(TypeError, match=r"global_step_transform should be a function."):
         Checkpoint(to_save, lambda x: x, score_function=lambda e: 123, score_name="acc", global_step_transform=123)
 
-    with pytest.warns(UserWarning, match=r"Argument archived is deprecated"):
-        Checkpoint(to_save, lambda x: x, score_function=lambda e: 123, score_name="acc", archived=True)
-
     with pytest.raises(ValueError, match=r"Cannot have key 'checkpointer' if `include_self` is True"):
         Checkpoint({"checkpointer": model}, lambda x: x, include_self=True)
 
     class ImmutableMapping(Mapping):
+        def __init__(self, d):
+            self._dict = d
+
         def __getitem__(self, key):
-            return to_save[key]
+            return self._dict[key]
 
         def __iter__(self):
-            return iter(to_save)
+            return iter(self._dict)
 
         def __len__(self):
-            return len(to_save)
+            return len(self._dict)
 
     with pytest.raises(TypeError, match="If `include_self` is True, then `to_save` must be mutable"):
-        Checkpoint(ImmutableMapping(), lambda x: x, include_self=True)
+        Checkpoint(ImmutableMapping(to_save), lambda x: x, include_self=True)
+
+    checkpoint = Checkpoint(to_save, lambda x: x)
+    with pytest.raises(AttributeError, match="Checkpoint's `save_handler` should be of type `DiskSaver`"):
+        checkpoint.reload_objects(to_save)
 
 
 def test_save_handler_as_str(dirname):
-
     to_save = {"model": model}
 
     checkpointer = Checkpoint(to_save, save_handler=dirname)
@@ -92,7 +97,6 @@ def test_save_handler_as_str(dirname):
 
 
 def test_checkpoint_score_function_wrong_output():
-
     to_save = {"model": model}
 
     checkpointer = Checkpoint(to_save, lambda x: x, score_function=lambda e: {"1": 1}, score_name="acc")
@@ -163,7 +167,7 @@ def test_checkpoint_include_self_state_dict(to_save, obj, name):
     assert save_handler.call_count == 1
 
     fname = f"{name}_0.pt"
-    obj["checkpointer"] = OrderedDict([("saved", [(0, fname)])])
+    obj["checkpointer"] = OrderedDict([("_saved", [(0, fname)])])
 
     metadata = {"basename": name, "score_name": None, "priority": 0}
     save_handler.assert_called_with(obj, fname, metadata)
@@ -183,7 +187,7 @@ def test_checkpoint_include_self_state_dict(to_save, obj, name):
     save_handler.remove.assert_called_with(f"{name}_0.pt")
 
     fname = f"{name}_1234.pt"
-    obj["checkpointer"] = OrderedDict([("saved", [(1234, fname)])])
+    obj["checkpointer"] = OrderedDict([("_saved", [(1234, fname)])])
 
     save_handler.assert_called_with(obj, fname, metadata)
     assert save_handler.remove.call_count == 1
@@ -191,7 +195,6 @@ def test_checkpoint_include_self_state_dict(to_save, obj, name):
 
 
 def test_checkpoint_with_dp():
-
     dp_model = nn.DataParallel(model)
     to_save = {"model": dp_model}
 
@@ -553,21 +556,12 @@ def test_model_checkpoint_args_validation(dirname):
     with pytest.raises(ValueError, match=r"with extension '.pt' are already present "):
         ModelCheckpoint(nonempty, _PREFIX)
 
-    with pytest.raises(ValueError, match=r"Argument save_interval is deprecated and should be None"):
-        ModelCheckpoint(existing, _PREFIX, save_interval=42)
-
     with pytest.raises(ValueError, match=r"Directory path '\S+' is not found"):
         ModelCheckpoint(dirname / "non_existing_dir", _PREFIX, create_dir=False)
 
-    with pytest.raises(ValueError, match=r"Argument save_as_state_dict is deprecated and should be True"):
-        ModelCheckpoint(existing, _PREFIX, create_dir=False, save_as_state_dict=False)
-
     with pytest.raises(TypeError, match=r"global_step_transform should be a function"):
         ModelCheckpoint(existing, _PREFIX, create_dir=False, global_step_transform=1234)
 
-    with pytest.warns(UserWarning, match=r"Argument archived is deprecated"):
-        ModelCheckpoint(existing, _PREFIX, create_dir=False, archived=True)
-
     h = ModelCheckpoint(dirname, _PREFIX, create_dir=False)
     assert h.last_checkpoint is None
     with pytest.raises(RuntimeError, match=r"No objects to checkpoint found."):
@@ -596,7 +590,6 @@ def test_model_checkpoint_simple_recovery(dirname):
 
 @pytest.mark.parametrize("ext, require_empty", [(".txt", True), (".pt", False)])
 def test_model_checkpoint_simple_recovery_from_existing_non_empty(ext, require_empty, dirname):
-
     previous_fname = dirname / f"{_PREFIX}_obj_{1}{ext}"
     with open(previous_fname, "w") as f:
         f.write("test")
@@ -636,13 +629,11 @@ def test_model_checkpoint_invalid_save_handler(dirname):
 
 
 def test_disk_saver_atomic(dirname):
-
     model = DummyModel()
     to_save_serializable = {"model": model}
     to_save_non_serializable = {"model": lambda x: x}
 
     def _test_existence(atomic, _to_save, expected):
-
         saver = DiskSaver(dirname, atomic=atomic, create_dir=False, require_empty=False)
         fname = "test.pt"
         try:
@@ -697,7 +688,6 @@ def test_disk_saver_unknown_keyword(dirname):
 
 
 def test_last_k(dirname):
-
     h = ModelCheckpoint(dirname, _PREFIX, create_dir=False, n_saved=2)
     engine = Engine(lambda e, b: None)
     engine.state = State(epoch=0, iteration=0)
@@ -716,7 +706,6 @@ def test_last_k(dirname):
 
 
 def test_disabled_n_saved(dirname):
-
     h = ModelCheckpoint(dirname, _PREFIX, create_dir=False, n_saved=None)
     engine = Engine(lambda e, b: None)
     engine.state = State(epoch=0, iteration=0)
@@ -869,7 +858,6 @@ def test_valid_state_dict_save(dirname):
 
 
 def _test_save_model_optimizer_lr_scheduler_with_state_dict(device, dirname, just_on_zero_rank=False):
-
     torch.manual_seed(23)
 
     model = DummyModel().to(device)
@@ -885,7 +873,6 @@ def _test_save_model_optimizer_lr_scheduler_with_state_dict(device, dirname, jus
         # Probably related to https://github.com/pytorch/xla/issues/2576
         # loss = y.pow(2.0).sum()
         loss = y.sum()
-        print(loss.device, y.device, x.device)
         loss.backward()
         if idist.has_xla_support:
             import torch_xla.core.xla_model as xm
@@ -958,7 +945,6 @@ def _test_save_model_optimizer_lr_scheduler_with_validation(device, dirname, jus
     torch.manual_seed(23)
 
     def _build_objects(acc_list):
-
         model = DummyModel().to(device)
         optim = torch.optim.SGD(model.parameters(), lr=0.1)
         lr_scheduler = torch.optim.lr_scheduler.ExponentialLR(optim, gamma=0.5)
@@ -1087,17 +1073,19 @@ def test_save_model_optimizer_lr_scheduler_with_validation(dirname):
 
 
 def test_checkpoint_load_objects():
-
     with pytest.raises(TypeError, match=r"Argument checkpoint should be a string or a dictionary"):
         Checkpoint.load_objects({}, [])
 
     with pytest.raises(TypeError, match=r"should have `load_state_dict` method"):
         Checkpoint.load_objects({"a": None}, {"a": None})
 
+    with pytest.raises(TypeError, match=r"should have `load_state_dict` method"):
+        Checkpoint.load_objects({"a": {"b": None}}, {"a": {"b": None}})
+
     model = DummyModel()
     to_load = {"model": model, "another_model": model}
 
-    with pytest.raises(ValueError, match=r"from `to_load` is not found in the checkpoint"):
+    with pytest.raises(ValueError, match=r"Key 'model' from x is not found in y"):
         Checkpoint.load_objects(to_load, {})
 
     model = DummyModel()
@@ -1108,6 +1096,11 @@ def test_checkpoint_load_objects():
     Checkpoint.load_objects(to_load, chkpt)
     assert model.state_dict() == model2.state_dict()
 
+    chkpt = {"models": [{"model1": {"abc": model.state_dict()}}, model.state_dict()]}
+    to_load = {"models": [{"model1": {"abc": model}}, model]}
+    Checkpoint.load_objects(to_load, chkpt)
+    assert model.state_dict() == model2.state_dict()
+
 
 def test_checkpoint_load_objects_from_saved_file(dirname):
     def _get_single_obj_to_save():
@@ -1119,7 +1112,11 @@ def test_checkpoint_load_objects_from_saved_file(dirname):
         model = DummyModel()
         optim = torch.optim.SGD(model.parameters(), lr=0.001)
         lr_scheduler = torch.optim.lr_scheduler.ExponentialLR(optim, gamma=0.5)
-        to_save = {"model": model, "optimizer": optim, "lr_scheduler": lr_scheduler}
+        to_save = {
+            "model": model,
+            "optimizer": optim,
+            "lr_scheduler": lr_scheduler,
+        }
         return to_save
 
     trainer = Engine(lambda e, b: None)
@@ -1193,9 +1190,7 @@ def test_load_checkpoint_with_different_num_classes(dirname):
     with pytest.raises(RuntimeError):
         Checkpoint.load_objects(to_load_single_object, loaded_checkpoint)
 
-    with warnings.catch_warnings():
-        warnings.simplefilter("ignore", category=UserWarning)
-        Checkpoint.load_objects(to_load_single_object, loaded_checkpoint, strict=False, blah="blah")
+    Checkpoint.load_objects(to_load_single_object, loaded_checkpoint, strict=False)
 
     loaded_weights = to_load_single_object["pretrained_features"].state_dict()["weight"]
 
@@ -1203,7 +1198,6 @@ def test_load_checkpoint_with_different_num_classes(dirname):
 
 
 def test_disksaver_wrong_input(dirname):
-
     with pytest.raises(ValueError, match=r"Directory path '\S+' is not found"):
         DiskSaver("/tmp/non-existing-folder", create_dir=False)
 
@@ -1256,7 +1250,6 @@ def _test_checkpoint_load_objects_ddp(device):
 
 
 def _test_checkpoint_with_ZeRO(device, dirname, local_rank):
-
     from torch.distributed.optim import ZeroRedundancyOptimizer
 
     model = DummyModel().to(device)
@@ -1277,7 +1270,6 @@ def _test_checkpoint_with_ZeRO(device, dirname, local_rank):
     mocked_opt.consolidate_state_dict.assert_called_once_with(to=1)
 
     if local_rank == 1:
-
         loaded_state_dict = torch.load(dirname / "checkpoint_0.pt", map_location=device)["optim"]
         state_dict = opt.state_dict()
         assert loaded_state_dict == state_dict
@@ -1286,7 +1278,6 @@ def _test_checkpoint_with_ZeRO(device, dirname, local_rank):
 @pytest.mark.distributed
 @pytest.mark.skipif(not idist.has_native_dist_support, reason="Skip if no native dist support")
 def test_distrib_gloo_cpu_or_gpu(distributed_context_single_node_gloo, dirname, get_rank_zero_dirname, local_rank):
-
     device = idist.device()
     rank_zero_dirname = get_rank_zero_dirname()
     _test_save_model_optimizer_lr_scheduler_with_state_dict(device, rank_zero_dirname / "1")
@@ -1304,7 +1295,6 @@ def test_distrib_gloo_cpu_or_gpu(distributed_context_single_node_gloo, dirname,
 @pytest.mark.skipif(not idist.has_native_dist_support, reason="Skip if no native dist support")
 @pytest.mark.skipif(torch.cuda.device_count() < 1, reason="Skip if no GPU")
 def test_distrib_nccl_gpu(distributed_context_single_node_nccl, get_rank_zero_dirname):
-
     device = idist.device()
     dirname = get_rank_zero_dirname()
     _test_save_model_optimizer_lr_scheduler_with_state_dict(device, dirname / "1")
@@ -1317,7 +1307,6 @@ def test_distrib_nccl_gpu(distributed_context_single_node_nccl, get_rank_zero_di
 @pytest.mark.skipif(not idist.has_hvd_support, reason="Skip if no Horovod dist support")
 @pytest.mark.skipif("WORLD_SIZE" in os.environ, reason="Skip if launched as multiproc")
 def test_distrib_hvd(gloo_hvd_executor, get_rank_zero_dirname):
-
     device = torch.device("cpu" if not torch.cuda.is_available() else "cuda")
     nproc = 4 if not torch.cuda.is_available() else torch.cuda.device_count()
     dirname = get_rank_zero_dirname()
@@ -1442,7 +1431,6 @@ def _test_model_checkpoint_filename_pattern_helper(
 
 @pytest.mark.parametrize("test_class", ["checkpoint", "model_checkpoint"])
 def test_checkpoint_filename_pattern(test_class, dirname):
-
     if test_class == "checkpoint":
         _test = _test_checkpoint_filename_pattern_helper
     elif test_class == "model_checkpoint":
@@ -1639,10 +1627,10 @@ def _setup_checkpoint():
 def test_checkpoint_state_dict():
     checkpointer = _setup_checkpoint()
     sd = checkpointer.state_dict()
-    assert "saved" in sd
-    assert isinstance(sd["saved"], list) and len(sd["saved"]) == len(checkpointer._saved)
+    assert "_saved" in sd
+    assert isinstance(sd["_saved"], list) and len(sd["_saved"]) == len(checkpointer._saved)
 
-    for saved_item, true_item in zip(sd["saved"], checkpointer._saved):
+    for saved_item, true_item in zip(sd["_saved"], checkpointer._saved):
         assert saved_item[0] == true_item.priority
         assert saved_item[1] == true_item.filename
 
@@ -1655,11 +1643,43 @@ def test_checkpoint_load_state_dict():
     to_save = {"model": model}
     checkpointer = Checkpoint(to_save, save_handler=save_handler, n_saved=None)
 
-    sd = {"saved": [(0, "model_0.pt"), (10, "model_10.pt"), (20, "model_20.pt")]}
+    sd = {"_saved": [(0, "model_0.pt"), (10, "model_10.pt"), (20, "model_20.pt")]}
     checkpointer.load_state_dict(sd)
     assert checkpointer._saved == true_checkpointer._saved
 
 
+@pytest.mark.parametrize(
+    "to_save",
+    [
+        {"model": DummyModel()},
+        {"model": [DummyModel(), DummyModel()]},
+        {"model": {"a": {"b": DummyModel()}}},
+    ],
+)
+def test_checkpoint__setup_checkpoint(to_save):
+    save_handler = MagicMock(spec=BaseSaveHandler)
+    checkpointer = Checkpoint(to_save, save_handler=save_handler, n_saved=2)
+    checkpoint = checkpointer._setup_checkpoint()
+
+    assert isinstance(checkpoint, dict)
+    for k, obj in to_save.items():
+        assert k in checkpoint
+        if isinstance(obj, torch.nn.Module):
+            assert checkpoint[k] == obj.state_dict()
+        elif isinstance(obj, list):
+            for c2, obj2 in zip(checkpoint[k], obj):
+                assert c2 == obj2.state_dict()
+        elif isinstance(obj, dict):
+            c2 = checkpoint[k]
+            for k2, obj2 in obj.items():
+                if isinstance(obj2, torch.nn.Module):
+                    assert c2[k2] == obj2.state_dict()
+                elif isinstance(obj2, dict):
+                    c3 = c2[k2]
+                    for k3, obj3 in obj2.items():
+                        assert c3[k3] == obj3.state_dict()
+
+
 def test_checkpoint_fixed_filename():
     model = DummyModel()
     to_save = {"model": model}
@@ -1795,7 +1815,6 @@ def test_greater_or_equal_model_checkpoint(dirname):
 
 
 def test_get_default_score_fn():
-
     with pytest.raises(ValueError, match=r"Argument score_sign should be 1 or -1"):
         Checkpoint.get_default_score_fn("acc", 2.0)
 
@@ -1835,7 +1854,6 @@ def test_load_single_object(obj_to_save, dirname):
 @pytest.mark.skipif(not idist.has_native_dist_support, reason="Skip if no native dist support")
 @pytest.mark.parametrize("atomic", [False, True])
 def test_disksaver_distrib(distributed_context_single_node_gloo, dirname, local_rank, atomic):
-
     saver = DiskSaver(dirname, atomic, save_on_rank=1)
     mocked_saver = MagicMock(wraps=saver)
 
