@@ -3,6 +3,8 @@ import os
 from pathlib import Path
 from unittest.mock import MagicMock
 
+import filelock
+
 import matplotlib
 import pytest
 import torch
@@ -11,8 +13,8 @@ from torch import nn
 from torch.optim import SGD
 
 import ignite.distributed as idist
-from ignite.contrib.handlers import FastaiLRFinder
 from ignite.engine import create_supervised_trainer, Engine, Events
+from ignite.handlers import FastaiLRFinder
 
 matplotlib.use("agg")
 
@@ -144,22 +146,32 @@ def dataloader_plot():
 
 
 @pytest.fixture
-def mnist_dataloader():
+def mnist_dataloader(tmp_path_factory):
     from torch.utils.data import DataLoader
     from torchvision.datasets import MNIST
     from torchvision.transforms import Compose, Normalize, ToTensor
 
     data_transform = Compose([ToTensor(), Normalize((0.1307,), (0.3081,))])
 
-    train_loader = DataLoader(
-        MNIST(download=True, root="/tmp", transform=data_transform, train=True), batch_size=256, shuffle=True
-    )
+    root_tmp_dir = tmp_path_factory.getbasetemp().parent
+    while True:
+        try:
+            with filelock.FileLock(root_tmp_dir / "mnist_download.lock", timeout=0.2) as fn:
+                fn.acquire()
+                train_loader = DataLoader(
+                    MNIST(download=True, root="/tmp", transform=data_transform, train=True),
+                    batch_size=256,
+                    shuffle=True,
+                )
+                fn.release()
+                break
+        except filelock._error.Timeout:
+            pass
 
     yield train_loader
 
 
 def test_attach_incorrect_input_args(lr_finder, dummy_engine, model, optimizer, dataloader):
-
     with pytest.raises(TypeError, match=r"Argument to_save should be a mapping"):
         with lr_finder.attach(dummy_engine, to_save=123):
             pass
@@ -249,7 +261,6 @@ def test_with_attach(lr_finder, to_save, dummy_engine, dataloader):
 def test_wrong_values_start_lr_and_end_lr(
     lr_finder, dummy_engine, to_save, dummy_engine_mulitple_param_groups, to_save_mulitple_param_groups
 ):
-
     with pytest.raises(ValueError, match=r"start_lr must be less than end_lr"):
         with lr_finder.attach(dummy_engine, to_save=to_save, start_lr=10.0, end_lr=1.0):
             pass
@@ -322,7 +333,6 @@ def assert_output_sizes(lr_finder, dummy_engine):
 
 
 def test_num_iter_is_none(lr_finder, to_save, dummy_engine, dataloader):
-
     with pytest.warns(UserWarning, match=r"Run completed without loss diverging"):
         with lr_finder.attach(dummy_engine, to_save=to_save, diverge_th=float("inf")) as trainer_with_finder:
             trainer_with_finder.run(dataloader)
@@ -331,7 +341,6 @@ def test_num_iter_is_none(lr_finder, to_save, dummy_engine, dataloader):
 
 
 def test_num_iter_is_enough(lr_finder, to_save, dummy_engine, dataloader):
-
     with pytest.warns(UserWarning, match=r"Run completed without loss diverging"):
         with lr_finder.attach(
             dummy_engine, to_save=to_save, num_iter=50, diverge_th=float("inf")
@@ -409,13 +418,13 @@ def test_engine_output_type(lr_finder, dummy_engine, optimizer):
     lr_finder._history = {"lr": [], "loss": []}
     lr_finder._log_lr_and_loss(dummy_engine, output_transform=lambda x: x, smooth_f=0, diverge_th=1)
     loss = lr_finder._history["loss"][-1]
-    assert type(loss) == float
+    assert type(loss) is float
 
     dummy_engine.state.output = torch.tensor([10.0], dtype=torch.float32)
     lr_finder._history = {"lr": [], "loss": []}
     lr_finder._log_lr_and_loss(dummy_engine, output_transform=lambda x: x, smooth_f=0, diverge_th=1)
     loss = lr_finder._history["loss"][-1]
-    assert type(loss) == float
+    assert type(loss) is float
 
 
 def test_lr_suggestion_unexpected_curve(lr_finder, to_save, dummy_engine, dataloader):
@@ -460,11 +469,9 @@ def test_lr_suggestion_multiple_param_groups(lr_finder):
 
 
 def test_lr_suggestion_mnist(lr_finder, mnist_to_save, dummy_engine_mnist, mnist_dataloader):
-
     max_iters = 50
 
     with lr_finder.attach(dummy_engine_mnist, mnist_to_save, diverge_th=2, step_mode="linear") as trainer_with_finder:
-
         with trainer_with_finder.add_event_handler(
             Events.ITERATION_COMPLETED(once=max_iters), lambda _: trainer_with_finder.terminate()
         ):
@@ -476,7 +483,6 @@ def test_lr_suggestion_mnist(lr_finder, mnist_to_save, dummy_engine_mnist, mnist
 def test_apply_suggested_lr_unmatched_optimizers(
     lr_finder, mnist_to_save, dummy_engine_mnist, optimizer_multiple_param_groups, mnist_dataloader
 ):
-
     with lr_finder.attach(dummy_engine_mnist, mnist_to_save) as trainer_with_finder:
         trainer_with_finder.run(mnist_dataloader)
 
@@ -489,7 +495,6 @@ def test_apply_suggested_lr_unmatched_optimizers(
 def test_apply_suggested_lr_single_param_groups(
     lr_finder, mnist_to_save, dummy_engine_mnist, mnist_optimizer, mnist_dataloader
 ):
-
     with lr_finder.attach(dummy_engine_mnist, mnist_to_save) as trainer_with_finder:
         trainer_with_finder.run(mnist_dataloader)
 
@@ -506,7 +511,6 @@ def test_apply_suggested_lr_multiple_param_groups(
     optimizer_multiple_param_groups,
     dataloader_plot,
 ):
-
     with lr_finder.attach(dummy_engine_mulitple_param_groups, to_save_mulitple_param_groups) as trainer_with_finder:
         trainer_with_finder.run(dataloader_plot)
 
@@ -518,13 +522,11 @@ def test_apply_suggested_lr_multiple_param_groups(
 
 
 def test_no_matplotlib(no_site_packages, lr_finder):
-
     with pytest.raises(ModuleNotFoundError, match=r"This method requires matplotlib to be installed"):
         lr_finder.plot()
 
 
 def test_plot_single_param_group(dirname, lr_finder, mnist_to_save, dummy_engine_mnist, mnist_dataloader):
-
     with lr_finder.attach(dummy_engine_mnist, mnist_to_save, end_lr=20.0, smooth_f=0.04) as trainer_with_finder:
         trainer_with_finder.run(mnist_dataloader)
 
@@ -553,7 +555,6 @@ def test_plot_single_param_group(dirname, lr_finder, mnist_to_save, dummy_engine
 def test_plot_multiple_param_groups(
     dirname, lr_finder, to_save_mulitple_param_groups, dummy_engine_mulitple_param_groups, dataloader_plot
 ):
-
     with lr_finder.attach(
         dummy_engine_mulitple_param_groups, to_save_mulitple_param_groups, end_lr=20.0, smooth_f=0.04
     ) as trainer_with_finder:
@@ -654,7 +655,6 @@ def _test_distrib_integration_mnist(dirname, device):
 @pytest.mark.distributed
 @pytest.mark.skipif(not idist.has_native_dist_support, reason="Skip if no native dist support")
 def test_distrib_gloo_cpu_or_gpu(dirname, distributed_context_single_node_gloo):
-
     device = idist.device()
     _test_distrib_log_lr_and_loss(device)
     _test_distrib_integration_mnist(dirname, device)
@@ -664,7 +664,6 @@ def test_distrib_gloo_cpu_or_gpu(dirname, distributed_context_single_node_gloo):
 @pytest.mark.skipif(not idist.has_native_dist_support, reason="Skip if no native dist support")
 @pytest.mark.skipif(torch.cuda.device_count() < 1, reason="Skip if no GPU")
 def test_distrib_nccl_gpu(dirname, distributed_context_single_node_nccl):
-
     device = idist.device()
     _test_distrib_log_lr_and_loss(device)
     _test_distrib_integration_mnist(dirname, device)
