@@ -6,6 +6,7 @@ from sklearn.exceptions import UndefinedMetricWarning
 from sklearn.metrics import precision_score
 
 import ignite.distributed as idist
+from ignite.engine import Engine
 from ignite.exceptions import NotComputableError
 from ignite.metrics import Precision
 
@@ -102,7 +103,6 @@ def ignite_average_to_scikit_average(average, data_type: str):
 
 @pytest.mark.parametrize("average", [None, False, "macro", "micro", "weighted"])
 def test_binary_input(average):
-
     pr = Precision(average=average)
     assert pr._updated is False
 
@@ -131,7 +131,6 @@ def test_binary_input(average):
         ) == pytest.approx(pr_compute)
 
     def get_test_cases():
-
         test_cases = [
             # Binary accuracy on input of shape (N, 1) or (N, )
             (torch.randint(0, 2, size=(10,)), torch.randint(0, 2, size=(10,)), 1),
@@ -224,7 +223,6 @@ def test_multiclass_wrong_inputs():
 
 @pytest.mark.parametrize("average", [None, False, "macro", "micro", "weighted"])
 def test_multiclass_input(average):
-
     pr = Precision(average=average)
     assert pr._updated is False
 
@@ -255,7 +253,6 @@ def test_multiclass_input(average):
             assert sk_compute == pytest.approx(pr_compute)
 
     def get_test_cases():
-
         test_cases = [
             # Multiclass input data of shape (N, ) and (N, C)
             (torch.rand(10, 6), torch.randint(0, 6, size=(10,)), 1),
@@ -325,7 +322,6 @@ def to_numpy_multilabel(y):
 
 @pytest.mark.parametrize("average", [None, False, "macro", "micro", "weighted", "samples"])
 def test_multilabel_input(average):
-
     pr = Precision(average=average, is_multilabel=True)
     assert pr._updated is False
 
@@ -353,7 +349,6 @@ def test_multilabel_input(average):
             assert precision_score(np_y, np_y_pred, average=sk_average_parameter) == pytest.approx(pr_compute)
 
     def get_test_cases():
-
         test_cases = [
             # Multilabel input data of shape (N, C)
             (torch.randint(0, 2, size=(10, 5)), torch.randint(0, 2, size=(10, 5)), 1),
@@ -423,212 +418,181 @@ def test_incorrect_y_classes(average):
     assert pr._updated is False
 
 
-def test_distrib_integration_multiclass(distributed):
-    from ignite.engine import Engine
+@pytest.mark.usefixtures("distributed")
+class TestDistributed:
+    @pytest.mark.parametrize("average", [False, "macro", "weighted", "micro"])
+    @pytest.mark.parametrize("n_epochs", [1, 2])
+    def test_integration_multiclass(self, average, n_epochs):
+        rank = idist.get_rank()
+        torch.manual_seed(12 + rank)
 
-    rank = idist.get_rank()
-    torch.manual_seed(12)
-
-    def _test(average, n_epochs, metric_device):
         n_iters = 60
-        s = 16
+        batch_size = 16
         n_classes = 7
 
-        offset = n_iters * s
-        y_true = torch.randint(0, n_classes, size=(offset * idist.get_world_size(),)).to(device)
-        y_preds = torch.rand(offset * idist.get_world_size(), n_classes).to(device)
+        metric_devices = [torch.device("cpu")]
+        device = idist.device()
+        if device.type != "xla":
+            metric_devices.append(idist.device())
 
-        def update(engine, i):
-            return (
-                y_preds[i * s + rank * offset : (i + 1) * s + rank * offset, :],
-                y_true[i * s + rank * offset : (i + 1) * s + rank * offset],
+        for metric_device in metric_devices:
+            y_true = torch.randint(0, n_classes, size=(n_iters * batch_size,)).to(device)
+            y_preds = torch.rand(n_iters * batch_size, n_classes).to(device)
+
+            def update(engine, i):
+                return (
+                    y_preds[i * batch_size : (i + 1) * batch_size, :],
+                    y_true[i * batch_size : (i + 1) * batch_size],
+                )
+
+            engine = Engine(update)
+
+            pr = Precision(average=average, device=metric_device)
+            pr.attach(engine, "pr")
+            assert pr._updated is False
+
+            data = list(range(n_iters))
+            engine.run(data=data, max_epochs=n_epochs)
+
+            y_preds = idist.all_gather(y_preds)
+            y_true = idist.all_gather(y_true)
+
+            assert "pr" in engine.state.metrics
+            assert pr._updated is True
+            res = engine.state.metrics["pr"]
+            if isinstance(res, torch.Tensor):
+                # Fixes https://github.com/pytorch/ignite/issues/1635#issuecomment-863026919
+                assert res.device.type == "cpu"
+                res = res.cpu().numpy()
+
+            sk_average_parameter = ignite_average_to_scikit_average(average, "multiclass")
+            true_res = precision_score(
+                y_true.cpu().numpy(), torch.argmax(y_preds, dim=1).cpu().numpy(), average=sk_average_parameter
             )
 
-        engine = Engine(update)
+            assert pytest.approx(res) == true_res
 
-        pr = Precision(average=average, device=metric_device)
-        pr.attach(engine, "pr")
-        assert pr._updated is False
+    @pytest.mark.parametrize("average", [False, "macro", "weighted", "micro", "samples"])
+    @pytest.mark.parametrize("n_epochs", [1, 2])
+    def test_integration_multilabel(self, average, n_epochs):
+        rank = idist.get_rank()
+        torch.manual_seed(12 + rank)
 
-        data = list(range(n_iters))
-        engine.run(data=data, max_epochs=n_epochs)
-
-        assert "pr" in engine.state.metrics
-        assert pr._updated is True
-        res = engine.state.metrics["pr"]
-        if isinstance(res, torch.Tensor):
-            # Fixes https://github.com/pytorch/ignite/issues/1635#issuecomment-863026919
-            assert res.device.type == "cpu"
-            res = res.cpu().numpy()
-
-        sk_average_parameter = ignite_average_to_scikit_average(average, "multiclass")
-        true_res = precision_score(
-            y_true.cpu().numpy(), torch.argmax(y_preds, dim=1).cpu().numpy(), average=sk_average_parameter
-        )
-
-        assert pytest.approx(res) == true_res
-
-    metric_devices = [torch.device("cpu")]
-    device = idist.device()
-    if device.type != "xla":
-        metric_devices.append(idist.device())
-    for _ in range(2):
-        for metric_device in metric_devices:
-            _test(average=False, n_epochs=1, metric_device=metric_device)
-            _test(average=False, n_epochs=2, metric_device=metric_device)
-            _test(average="macro", n_epochs=1, metric_device=metric_device)
-            _test(average="macro", n_epochs=2, metric_device=metric_device)
-            _test(average="weighted", n_epochs=1, metric_device=metric_device)
-            _test(average="weighted", n_epochs=2, metric_device=metric_device)
-            _test(average="micro", n_epochs=1, metric_device=metric_device)
-            _test(average="micro", n_epochs=2, metric_device=metric_device)
-
-
-def test_distrib_integration_multilabel(distributed):
-
-    from ignite.engine import Engine
-
-    rank = idist.get_rank()
-    torch.manual_seed(12)
-
-    def _test(average, n_epochs, metric_device):
         n_iters = 60
-        s = 16
+        batch_size = 16
         n_classes = 7
 
-        offset = n_iters * s
-        y_true = torch.randint(0, 2, size=(offset * idist.get_world_size(), n_classes, 6, 8)).to(device)
-        y_preds = torch.randint(0, 2, size=(offset * idist.get_world_size(), n_classes, 6, 8)).to(device)
+        metric_devices = ["cpu"]
+        device = idist.device()
+        if device.type != "xla":
+            metric_devices.append(idist.device())
 
-        def update(engine, i):
-            return (
-                y_preds[i * s + rank * offset : (i + 1) * s + rank * offset, ...],
-                y_true[i * s + rank * offset : (i + 1) * s + rank * offset, ...],
-            )
-
-        engine = Engine(update)
-
-        pr = Precision(average=average, is_multilabel=True, device=metric_device)
-        pr.attach(engine, "pr")
-        assert pr._updated is False
-
-        data = list(range(n_iters))
-        engine.run(data=data, max_epochs=n_epochs)
-
-        assert "pr" in engine.state.metrics
-        assert pr._updated is True
-        res = engine.state.metrics["pr"]
-        res2 = pr.compute()
-        if isinstance(res, torch.Tensor):
-            res = res.cpu().numpy()
-            res2 = res2.cpu().numpy()
-            assert (res == res2).all()
-        else:
-            assert res == res2
-
-        np_y_preds = to_numpy_multilabel(y_preds)
-        np_y_true = to_numpy_multilabel(y_true)
-        assert pr._type == "multilabel"
-        sk_average_parameter = ignite_average_to_scikit_average(average, "multilabel")
-        with warnings.catch_warnings():
-            warnings.simplefilter("ignore", category=UndefinedMetricWarning)
-            assert precision_score(np_y_true, np_y_preds, average=sk_average_parameter) == pytest.approx(res)
-
-    metric_devices = ["cpu"]
-    device = idist.device()
-    if device.type != "xla":
-        metric_devices.append(idist.device())
-    for _ in range(2):
         for metric_device in metric_devices:
-            _test(average=False, n_epochs=1, metric_device=metric_device)
-            _test(average=False, n_epochs=2, metric_device=metric_device)
-            _test(average="macro", n_epochs=1, metric_device=metric_device)
-            _test(average="macro", n_epochs=2, metric_device=metric_device)
-            _test(average="micro", n_epochs=1, metric_device=metric_device)
-            _test(average="micro", n_epochs=2, metric_device=metric_device)
-            _test(average="weighted", n_epochs=1, metric_device=metric_device)
-            _test(average="weighted", n_epochs=2, metric_device=metric_device)
-            _test(average="samples", n_epochs=1, metric_device=metric_device)
-            _test(average="samples", n_epochs=2, metric_device=metric_device)
+            y_true = torch.randint(0, 2, size=(n_iters * batch_size, n_classes, 6, 8)).to(device)
+            y_preds = torch.randint(0, 2, size=(n_iters * batch_size, n_classes, 6, 8)).to(device)
 
+            def update(engine, i):
+                return (
+                    y_preds[i * batch_size : (i + 1) * batch_size, ...],
+                    y_true[i * batch_size : (i + 1) * batch_size, ...],
+                )
 
-def test_distrib_accumulator_device(distributed):
-    # Binary accuracy on input of shape (N, 1) or (N, )
+            engine = Engine(update)
 
-    def _test(average, metric_device):
-        pr = Precision(average=average, device=metric_device)
-        assert pr._device == metric_device
-        assert pr._updated is False
-        # Since the shape of the accumulated amount isn't known before the first update
-        # call, the internal variables aren't tensors on the right device yet.
+            pr = Precision(average=average, is_multilabel=True, device=metric_device)
+            pr.attach(engine, "pr")
+            assert pr._updated is False
 
-        y_pred = torch.randint(0, 2, size=(10,))
-        y = torch.randint(0, 2, size=(10,)).long()
-        pr.update((y_pred, y))
+            data = list(range(n_iters))
+            engine.run(data=data, max_epochs=n_epochs)
 
-        assert pr._updated is True
+            y_preds = idist.all_gather(y_preds)
+            y_true = idist.all_gather(y_true)
 
-        assert (
-            pr._numerator.device == metric_device
-        ), f"{type(pr._numerator.device)}:{pr._numerator.device} vs {type(metric_device)}:{metric_device}"
+            assert "pr" in engine.state.metrics
+            assert pr._updated is True
+            res = engine.state.metrics["pr"]
+            res2 = pr.compute()
+            if isinstance(res, torch.Tensor):
+                res = res.cpu().numpy()
+                res2 = res2.cpu().numpy()
+                assert (res == res2).all()
+            else:
+                assert res == res2
+
+            np_y_preds = to_numpy_multilabel(y_preds)
+            np_y_true = to_numpy_multilabel(y_true)
+            assert pr._type == "multilabel"
+            sk_average_parameter = ignite_average_to_scikit_average(average, "multilabel")
+            with warnings.catch_warnings():
+                warnings.simplefilter("ignore", category=UndefinedMetricWarning)
+                assert precision_score(np_y_true, np_y_preds, average=sk_average_parameter) == pytest.approx(res)
+
+    @pytest.mark.parametrize("average", [False, "macro", "weighted", "micro"])
+    def test_accumulator_device(self, average):
+        # Binary accuracy on input of shape (N, 1) or (N, )
+
+        metric_devices = [torch.device("cpu")]
+        device = idist.device()
+        if device.type != "xla":
+            metric_devices.append(idist.device())
+
+        for metric_device in metric_devices:
+            pr = Precision(average=average, device=metric_device)
+            assert pr._device == metric_device
+            assert pr._updated is False
+            # Since the shape of the accumulated amount isn't known before the first update
+            # call, the internal variables aren't tensors on the right device yet.
+
+            y_pred = torch.randint(0, 2, size=(10,))
+            y = torch.randint(0, 2, size=(10,)).long()
+            pr.update((y_pred, y))
+
+            assert pr._updated is True
 
-        if average != "samples":
-            # For average='samples', `_denominator` is of type `int` so it has not `device` member.
             assert (
-                pr._denominator.device == metric_device
-            ), f"{type(pr._denominator.device)}:{pr._denominator.device} vs {type(metric_device)}:{metric_device}"
+                pr._numerator.device == metric_device
+            ), f"{type(pr._numerator.device)}:{pr._numerator.device} vs {type(metric_device)}:{metric_device}"
 
-        if average == "weighted":
-            assert pr._weight.device == metric_device, f"{type(pr._weight.device)}:{pr._weight.device} vs "
-            f"{type(metric_device)}:{metric_device}"
+            if average != "samples":
+                # For average='samples', `_denominator` is of type `int` so it has not `device` member.
+                assert (
+                    pr._denominator.device == metric_device
+                ), f"{type(pr._denominator.device)}:{pr._denominator.device} vs {type(metric_device)}:{metric_device}"
 
-    metric_devices = [torch.device("cpu")]
-    device = idist.device()
-    if device.type != "xla":
-        metric_devices.append(idist.device())
-    for metric_device in metric_devices:
-        _test(False, metric_device=metric_device)
-        _test("macro", metric_device=metric_device)
-        _test("micro", metric_device=metric_device)
-        _test("weighted", metric_device=metric_device)
+            if average == "weighted":
+                assert pr._weight.device == metric_device, f"{type(pr._weight.device)}:{pr._weight.device} vs "
+                f"{type(metric_device)}:{metric_device}"
 
+    @pytest.mark.parametrize("average", [False, "macro", "weighted", "micro", "samples"])
+    def test_multilabel_accumulator_device(self, average):
+        # Multiclass input data of shape (N, ) and (N, C)
 
-def test_distrib_multilabel_accumulator_device(distributed):
-    # Multiclass input data of shape (N, ) and (N, C)
+        metric_devices = [torch.device("cpu")]
+        device = idist.device()
+        if device.type != "xla":
+            metric_devices.append(idist.device())
+        for metric_device in metric_devices:
+            pr = Precision(is_multilabel=True, average=average, device=metric_device)
+            assert pr._device == metric_device
+            assert pr._updated is False
 
-    def _test(average, metric_device):
-        pr = Precision(is_multilabel=True, average=average, device=metric_device)
+            y_pred = torch.randint(0, 2, size=(10, 4, 20, 23))
+            y = torch.randint(0, 2, size=(10, 4, 20, 23)).long()
+            pr.update((y_pred, y))
 
-        assert pr._updated is False
-        assert pr._device == metric_device
+            assert pr._updated is True
 
-        y_pred = torch.randint(0, 2, size=(10, 4, 20, 23))
-        y = torch.randint(0, 2, size=(10, 4, 20, 23)).long()
-        pr.update((y_pred, y))
-
-        assert pr._updated is True
-
-        assert (
-            pr._numerator.device == metric_device
-        ), f"{type(pr._numerator.device)}:{pr._numerator.device} vs {type(metric_device)}:{metric_device}"
-
-        if average != "samples":
-            # For average='samples', `_denominator` is of type `int` so it has not `device` member.
             assert (
-                pr._denominator.device == metric_device
-            ), f"{type(pr._denominator.device)}:{pr._denominator.device} vs {type(metric_device)}:{metric_device}"
+                pr._numerator.device == metric_device
+            ), f"{type(pr._numerator.device)}:{pr._numerator.device} vs {type(metric_device)}:{metric_device}"
 
-        if average == "weighted":
-            assert pr._weight.device == metric_device, f"{type(pr._weight.device)}:{pr._weight.device} vs "
-            f"{type(metric_device)}:{metric_device}"
+            if average != "samples":
+                # For average='samples', `_denominator` is of type `int` so it has not `device` member.
+                assert (
+                    pr._denominator.device == metric_device
+                ), f"{type(pr._denominator.device)}:{pr._denominator.device} vs {type(metric_device)}:{metric_device}"
 
-    metric_devices = [torch.device("cpu")]
-    device = idist.device()
-    if device.type != "xla":
-        metric_devices.append(idist.device())
-    for metric_device in metric_devices:
-        _test(False, metric_device=metric_device)
-        _test("macro", metric_device=metric_device)
-        _test("micro", metric_device=metric_device)
-        _test("weighted", metric_device=metric_device)
-        _test("samples", metric_device=metric_device)
+            if average == "weighted":
+                assert pr._weight.device == metric_device, f"{type(pr._weight.device)}:{pr._weight.device} vs "
+                f"{type(metric_device)}:{metric_device}"
