@@ -1,4 +1,3 @@
-import os
 import warnings
 
 import pytest
@@ -7,6 +6,7 @@ from sklearn.exceptions import UndefinedMetricWarning
 from sklearn.metrics import recall_score
 
 import ignite.distributed as idist
+from ignite.engine import Engine
 from ignite.exceptions import NotComputableError
 from ignite.metrics import Recall
 
@@ -28,7 +28,6 @@ def test_no_update():
 
 
 def test_average_parameter():
-
     re = Recall(average="samples")
     with pytest.raises(
         ValueError, match=r"Argument average='samples' is incompatible with binary and multiclass input data."
@@ -107,7 +106,6 @@ def ignite_average_to_scikit_average(average, data_type: str):
 
 @pytest.mark.parametrize("average", [None, False, "macro", "micro", "weighted"])
 def test_binary_input(average):
-
     re = Recall(average=average)
     assert re._updated is False
 
@@ -134,7 +132,6 @@ def test_binary_input(average):
         assert recall_score(np_y, np_y_pred, average=sk_average_parameter, labels=[0, 1]) == pytest.approx(re_compute)
 
     def get_test_cases():
-
         test_cases = [
             # Binary accuracy on input of shape (N, 1) or (N, )
             (torch.randint(0, 2, size=(10,)), torch.randint(0, 2, size=(10,)), 1),
@@ -227,7 +224,6 @@ def test_multiclass_wrong_inputs():
 
 @pytest.mark.parametrize("average", [None, False, "macro", "micro", "weighted"])
 def test_multiclass_input(average):
-
     re = Recall(average=average)
     assert re._updated is False
 
@@ -258,7 +254,6 @@ def test_multiclass_input(average):
             assert sk_compute == pytest.approx(re_compute)
 
     def get_test_cases():
-
         test_cases = [
             # Multiclass input data of shape (N, ) and (N, C)
             (torch.rand(10, 6), torch.randint(0, 6, size=(10,)), 1),
@@ -328,7 +323,6 @@ def to_numpy_multilabel(y):
 
 @pytest.mark.parametrize("average", [None, False, "macro", "micro", "samples"])
 def test_multilabel_input(average):
-
     re = Recall(average=average, is_multilabel=True)
     assert re._updated is False
 
@@ -356,7 +350,6 @@ def test_multilabel_input(average):
             assert recall_score(np_y, np_y_pred, average=sk_average_parameter) == pytest.approx(re_compute)
 
     def get_test_cases():
-
         test_cases = [
             # Multilabel input data of shape (N, C)
             (torch.randint(0, 2, size=(10, 5)), torch.randint(0, 2, size=(10, 5)), 1),
@@ -426,301 +419,181 @@ def test_incorrect_y_classes(average):
     assert re._updated is False
 
 
-def _test_distrib_integration_multiclass(device):
+@pytest.mark.usefixtures("distributed")
+class TestDistributed:
+    @pytest.mark.parametrize("average", [False, "macro", "weighted", "micro"])
+    @pytest.mark.parametrize("n_epochs", [1, 2])
+    def test_integration_multiclass(self, average, n_epochs):
+        rank = idist.get_rank()
+        torch.manual_seed(12 + rank)
 
-    from ignite.engine import Engine
-
-    def _test(average, n_epochs, metric_device):
         n_iters = 60
         batch_size = 16
         n_classes = 7
 
-        y_true = torch.randint(0, n_classes, size=(n_iters * batch_size,)).to(device)
-        y_preds = torch.rand(n_iters * batch_size, n_classes).to(device)
+        metric_devices = [torch.device("cpu")]
+        device = idist.device()
+        if device.type != "xla":
+            metric_devices.append(idist.device())
 
-        def update(engine, i):
-            return (
-                y_preds[i * batch_size : (i + 1) * batch_size, :],
-                y_true[i * batch_size : (i + 1) * batch_size],
+        for metric_device in metric_devices:
+            y_true = torch.randint(0, n_classes, size=(n_iters * batch_size,)).to(device)
+            y_preds = torch.rand(n_iters * batch_size, n_classes).to(device)
+
+            def update(engine, i):
+                return (
+                    y_preds[i * batch_size : (i + 1) * batch_size, :],
+                    y_true[i * batch_size : (i + 1) * batch_size],
+                )
+
+            engine = Engine(update)
+
+            re = Recall(average=average, device=metric_device)
+            re.attach(engine, "re")
+            assert re._updated is False
+
+            data = list(range(n_iters))
+            engine.run(data=data, max_epochs=n_epochs)
+
+            y_preds = idist.all_gather(y_preds)
+            y_true = idist.all_gather(y_true)
+
+            assert "re" in engine.state.metrics
+            assert re._updated is True
+            res = engine.state.metrics["re"]
+            if isinstance(res, torch.Tensor):
+                # Fixes https://github.com/pytorch/ignite/issues/1635#issuecomment-863026919
+                assert res.device.type == "cpu"
+                res = res.cpu().numpy()
+
+            sk_average_parameter = ignite_average_to_scikit_average(average, "multiclass")
+            true_res = recall_score(
+                y_true.cpu().numpy(), torch.argmax(y_preds, dim=1).cpu().numpy(), average=sk_average_parameter
             )
 
-        engine = Engine(update)
+            assert pytest.approx(res) == true_res
 
-        re = Recall(average=average, device=metric_device)
-        re.attach(engine, "re")
-        assert re._updated is False
+    @pytest.mark.parametrize("average", [False, "macro", "weighted", "micro", "samples"])
+    @pytest.mark.parametrize("n_epochs", [1, 2])
+    def test_integration_multilabel(self, average, n_epochs):
+        rank = idist.get_rank()
+        torch.manual_seed(12 + rank)
 
-        data = list(range(n_iters))
-        engine.run(data=data, max_epochs=n_epochs)
-
-        y_preds = idist.all_gather(y_preds)
-        y_true = idist.all_gather(y_true)
-
-        assert "re" in engine.state.metrics
-        assert re._updated is True
-        res = engine.state.metrics["re"]
-        if isinstance(res, torch.Tensor):
-            # Fixes https://github.com/pytorch/ignite/issues/1635#issuecomment-863026919
-            assert res.device.type == "cpu"
-            res = res.cpu().numpy()
-
-        sk_average_parameter = ignite_average_to_scikit_average(average, "multiclass")
-        true_res = recall_score(
-            y_true.cpu().numpy(), torch.argmax(y_preds, dim=1).cpu().numpy(), average=sk_average_parameter
-        )
-
-        assert pytest.approx(res) == true_res
-
-    metric_devices = [torch.device("cpu")]
-    if device.type != "xla":
-        metric_devices.append(idist.device())
-    rank = idist.get_rank()
-    for i in range(2):
-        torch.manual_seed(12 + rank + i)
-        for metric_device in metric_devices:
-            _test(average=False, n_epochs=1, metric_device=metric_device)
-            _test(average=False, n_epochs=2, metric_device=metric_device)
-            _test(average="macro", n_epochs=1, metric_device=metric_device)
-            _test(average="macro", n_epochs=2, metric_device=metric_device)
-            _test(average="weighted", n_epochs=1, metric_device=metric_device)
-            _test(average="weighted", n_epochs=2, metric_device=metric_device)
-            _test(average="micro", n_epochs=1, metric_device=metric_device)
-            _test(average="micro", n_epochs=2, metric_device=metric_device)
-
-
-def _test_distrib_integration_multilabel(device):
-
-    from ignite.engine import Engine
-
-    torch.manual_seed(12)
-
-    def _test(average, n_epochs, metric_device):
         n_iters = 60
         batch_size = 16
         n_classes = 7
 
-        y_true = torch.randint(0, 2, size=(n_iters * batch_size, n_classes, 6, 8)).to(device)
-        y_preds = torch.randint(0, 2, size=(n_iters * batch_size, n_classes, 6, 8)).to(device)
+        metric_devices = ["cpu"]
+        device = idist.device()
+        if device.type != "xla":
+            metric_devices.append(idist.device())
 
-        def update(engine, i):
-            return (
-                y_preds[i * batch_size : (i + 1) * batch_size, ...],
-                y_true[i * batch_size : (i + 1) * batch_size, ...],
-            )
-
-        engine = Engine(update)
-
-        re = Recall(average=average, is_multilabel=True, device=metric_device)
-        re.attach(engine, "re")
-        assert re._updated is False
-
-        data = list(range(n_iters))
-        engine.run(data=data, max_epochs=n_epochs)
-
-        y_preds = idist.all_gather(y_preds)
-        y_true = idist.all_gather(y_true)
-
-        assert "re" in engine.state.metrics
-        assert re._updated is True
-        res = engine.state.metrics["re"]
-        res2 = re.compute()
-        if isinstance(res, torch.Tensor):
-            res = res.cpu().numpy()
-            res2 = res2.cpu().numpy()
-            assert (res == res2).all()
-        else:
-            assert res == res2
-
-        np_y_preds = to_numpy_multilabel(y_preds)
-        np_y_true = to_numpy_multilabel(y_true)
-        assert re._type == "multilabel"
-        sk_average_parameter = ignite_average_to_scikit_average(average, "multilabel")
-        with warnings.catch_warnings():
-            warnings.simplefilter("ignore", category=UndefinedMetricWarning)
-            assert recall_score(np_y_true, np_y_preds, average=sk_average_parameter) == pytest.approx(res)
-
-    metric_devices = ["cpu"]
-    if device.type != "xla":
-        metric_devices.append(idist.device())
-    rank = idist.get_rank()
-    for i in range(2):
-        torch.manual_seed(12 + rank + i)
         for metric_device in metric_devices:
-            _test(average=False, n_epochs=1, metric_device=metric_device)
-            _test(average=False, n_epochs=2, metric_device=metric_device)
-            _test(average="macro", n_epochs=1, metric_device=metric_device)
-            _test(average="macro", n_epochs=2, metric_device=metric_device)
-            _test(average="micro", n_epochs=1, metric_device=metric_device)
-            _test(average="micro", n_epochs=2, metric_device=metric_device)
-            _test(average="weighted", n_epochs=1, metric_device=metric_device)
-            _test(average="weighted", n_epochs=2, metric_device=metric_device)
-            _test(average="samples", n_epochs=1, metric_device=metric_device)
-            _test(average="samples", n_epochs=2, metric_device=metric_device)
+            y_true = torch.randint(0, 2, size=(n_iters * batch_size, n_classes, 6, 8)).to(device)
+            y_preds = torch.randint(0, 2, size=(n_iters * batch_size, n_classes, 6, 8)).to(device)
 
+            def update(engine, i):
+                return (
+                    y_preds[i * batch_size : (i + 1) * batch_size, ...],
+                    y_true[i * batch_size : (i + 1) * batch_size, ...],
+                )
 
-def _test_distrib_accumulator_device(device):
-    # Binary accuracy on input of shape (N, 1) or (N, )
+            engine = Engine(update)
 
-    def _test(average, metric_device):
-        re = Recall(average=average, device=metric_device)
-        assert re._device == metric_device
-        assert re._updated is False
-        # Since the shape of the accumulated amount isn't known before the first update
-        # call, the internal variables aren't tensors on the right device yet.
+            re = Recall(average=average, is_multilabel=True, device=metric_device)
+            re.attach(engine, "re")
+            assert re._updated is False
 
-        y_reed = torch.randint(0, 2, size=(10,))
-        y = torch.randint(0, 2, size=(10,)).long()
-        re.update((y_reed, y))
+            data = list(range(n_iters))
+            engine.run(data=data, max_epochs=n_epochs)
 
-        assert re._updated is True
+            y_preds = idist.all_gather(y_preds)
+            y_true = idist.all_gather(y_true)
 
-        assert (
-            re._numerator.device == metric_device
-        ), f"{type(re._numerator.device)}:{re._numerator.device} vs {type(metric_device)}:{metric_device}"
+            assert "re" in engine.state.metrics
+            assert re._updated is True
+            res = engine.state.metrics["re"]
+            res2 = re.compute()
+            if isinstance(res, torch.Tensor):
+                res = res.cpu().numpy()
+                res2 = res2.cpu().numpy()
+                assert (res == res2).all()
+            else:
+                assert res == res2
+
+            np_y_preds = to_numpy_multilabel(y_preds)
+            np_y_true = to_numpy_multilabel(y_true)
+            assert re._type == "multilabel"
+            sk_average_parameter = ignite_average_to_scikit_average(average, "multilabel")
+            with warnings.catch_warnings():
+                warnings.simplefilter("ignore", category=UndefinedMetricWarning)
+                assert recall_score(np_y_true, np_y_preds, average=sk_average_parameter) == pytest.approx(res)
+
+    @pytest.mark.parametrize("average", [False, "macro", "weighted", "micro"])
+    def test_accumulator_device(self, average):
+        # Binary accuracy on input of shape (N, 1) or (N, )
+
+        metric_devices = [torch.device("cpu")]
+        device = idist.device()
+        if device.type != "xla":
+            metric_devices.append(idist.device())
+        for metric_device in metric_devices:
+            re = Recall(average=average, device=metric_device)
+            assert re._device == metric_device
+            assert re._updated is False
+            # Since the shape of the accumulated amount isn't known before the first update
+            # call, the internal variables aren't tensors on the right device yet.
+
+            y_reed = torch.randint(0, 2, size=(10,))
+            y = torch.randint(0, 2, size=(10,)).long()
+            re.update((y_reed, y))
+
+            assert re._updated is True
 
-        if average != "samples":
-            # For average='samples', `_denominator` is of type `int` so it has not `device` member.
             assert (
-                re._denominator.device == metric_device
-            ), f"{type(re._denominator.device)}:{re._denominator.device} vs {type(metric_device)}:{metric_device}"
+                re._numerator.device == metric_device
+            ), f"{type(re._numerator.device)}:{re._numerator.device} vs {type(metric_device)}:{metric_device}"
 
-        if average == "weighted":
-            assert re._weight.device == metric_device, f"{type(re._weight.device)}:{re._weight.device} vs "
-            f"{type(metric_device)}:{metric_device}"
+            if average != "samples":
+                # For average='samples', `_denominator` is of type `int` so it has not `device` member.
+                assert (
+                    re._denominator.device == metric_device
+                ), f"{type(re._denominator.device)}:{re._denominator.device} vs {type(metric_device)}:{metric_device}"
 
-    metric_devices = [torch.device("cpu")]
-    if device.type != "xla":
-        metric_devices.append(idist.device())
-    for metric_device in metric_devices:
-        _test(False, metric_device=metric_device)
-        _test("macro", metric_device=metric_device)
-        _test("micro", metric_device=metric_device)
-        _test("weighted", metric_device=metric_device)
+            if average == "weighted":
+                assert re._weight.device == metric_device, f"{type(re._weight.device)}:{re._weight.device} vs "
+                f"{type(metric_device)}:{metric_device}"
 
+    @pytest.mark.parametrize("average", [False, "macro", "weighted", "micro", "samples"])
+    def test_multilabel_accumulator_device(self, average):
+        # Multiclass input data of shape (N, ) and (N, C)
 
-def _test_distrib_multilabel_accumulator_device(device):
-    # Multiclass input data of shape (N, ) and (N, C)
+        metric_devices = [torch.device("cpu")]
+        device = idist.device()
+        if device.type != "xla":
+            metric_devices.append(idist.device())
 
-    def _test(average, metric_device):
-        re = Recall(is_multilabel=True, average=average, device=metric_device)
+        for metric_device in metric_devices:
+            re = Recall(is_multilabel=True, average=average, device=metric_device)
+            assert re._device == metric_device
+            assert re._updated is False
 
-        assert re._updated is False
-        assert re._device == metric_device
+            y_reed = torch.randint(0, 2, size=(10, 4, 20, 23))
+            y = torch.randint(0, 2, size=(10, 4, 20, 23)).long()
+            re.update((y_reed, y))
 
-        y_reed = torch.randint(0, 2, size=(10, 4, 20, 23))
-        y = torch.randint(0, 2, size=(10, 4, 20, 23)).long()
-        re.update((y_reed, y))
+            assert re._updated is True
 
-        assert re._updated is True
-
-        assert (
-            re._numerator.device == metric_device
-        ), f"{type(re._numerator.device)}:{re._numerator.device} vs {type(metric_device)}:{metric_device}"
-
-        if average != "samples":
-            # For average='samples', `_denominator` is of type `int` so it has not `device` member.
             assert (
-                re._denominator.device == metric_device
-            ), f"{type(re._denominator.device)}:{re._denominator.device} vs {type(metric_device)}:{metric_device}"
+                re._numerator.device == metric_device
+            ), f"{type(re._numerator.device)}:{re._numerator.device} vs {type(metric_device)}:{metric_device}"
 
-        if average == "weighted":
-            assert re._weight.device == metric_device, f"{type(re._weight.device)}:{re._weight.device} vs "
-            f"{type(metric_device)}:{metric_device}"
+            if average != "samples":
+                # For average='samples', `_denominator` is of type `int` so it has not `device` member.
+                assert (
+                    re._denominator.device == metric_device
+                ), f"{type(re._denominator.device)}:{re._denominator.device} vs {type(metric_device)}:{metric_device}"
 
-    metric_devices = [torch.device("cpu")]
-    if device.type != "xla":
-        metric_devices.append(idist.device())
-    for metric_device in metric_devices:
-        _test(False, metric_device=metric_device)
-        _test("macro", metric_device=metric_device)
-        _test("micro", metric_device=metric_device)
-        _test("weighted", metric_device=metric_device)
-        _test("samples", metric_device=metric_device)
-
-
-@pytest.mark.distributed
-@pytest.mark.skipif(not idist.has_native_dist_support, reason="Skip if no native dist support")
-@pytest.mark.skipif(torch.cuda.device_count() < 1, reason="Skip if no GPU")
-def test_distrib_nccl_gpu(distributed_context_single_node_nccl):
-
-    device = idist.device()
-    _test_distrib_integration_multiclass(device)
-    _test_distrib_integration_multilabel(device)
-    _test_distrib_accumulator_device(device)
-    _test_distrib_multilabel_accumulator_device(device)
-
-
-@pytest.mark.distributed
-@pytest.mark.skipif(not idist.has_native_dist_support, reason="Skip if no native dist support")
-def test_distrib_gloo_cpu_or_gpu(distributed_context_single_node_gloo):
-
-    device = idist.device()
-    _test_distrib_integration_multiclass(device)
-    _test_distrib_integration_multilabel(device)
-    _test_distrib_accumulator_device(device)
-    _test_distrib_multilabel_accumulator_device(device)
-
-
-@pytest.mark.distributed
-@pytest.mark.skipif(not idist.has_hvd_support, reason="Skip if no Horovod dist support")
-@pytest.mark.skipif("WORLD_SIZE" in os.environ, reason="Skip if launched as multiproc")
-def test_distrib_hvd(gloo_hvd_executor):
-
-    device = torch.device("cpu" if not torch.cuda.is_available() else "cuda")
-    nproc = 4 if not torch.cuda.is_available() else torch.cuda.device_count()
-
-    gloo_hvd_executor(_test_distrib_integration_multiclass, (device,), np=nproc, do_init=True)
-    gloo_hvd_executor(_test_distrib_integration_multilabel, (device,), np=nproc, do_init=True)
-    gloo_hvd_executor(_test_distrib_accumulator_device, (device,), np=nproc, do_init=True)
-    gloo_hvd_executor(_test_distrib_multilabel_accumulator_device, (device,), np=nproc, do_init=True)
-
-
-@pytest.mark.multinode_distributed
-@pytest.mark.skipif(not idist.has_native_dist_support, reason="Skip if no native dist support")
-@pytest.mark.skipif("MULTINODE_DISTRIB" not in os.environ, reason="Skip if not multi-node distributed")
-def test_multinode_distrib_gloo_cpu_or_gpu(distributed_context_multi_node_gloo):
-
-    device = idist.device()
-    _test_distrib_integration_multiclass(device)
-    _test_distrib_integration_multilabel(device)
-    _test_distrib_accumulator_device(device)
-    _test_distrib_multilabel_accumulator_device(device)
-
-
-@pytest.mark.multinode_distributed
-@pytest.mark.skipif(not idist.has_native_dist_support, reason="Skip if no native dist support")
-@pytest.mark.skipif("GPU_MULTINODE_DISTRIB" not in os.environ, reason="Skip if not multi-node distributed")
-def test_multinode_distrib_nccl_gpu(distributed_context_multi_node_nccl):
-
-    device = idist.device()
-    _test_distrib_integration_multiclass(device)
-    _test_distrib_integration_multilabel(device)
-    _test_distrib_accumulator_device(device)
-    _test_distrib_multilabel_accumulator_device(device)
-
-
-@pytest.mark.tpu
-@pytest.mark.skipif("NUM_TPU_WORKERS" in os.environ, reason="Skip if NUM_TPU_WORKERS is in env vars")
-@pytest.mark.skipif(not idist.has_xla_support, reason="Skip if no PyTorch XLA package")
-def test_distrib_single_device_xla():
-    device = idist.device()
-    _test_distrib_integration_multiclass(device)
-    _test_distrib_integration_multilabel(device)
-    _test_distrib_accumulator_device(device)
-    _test_distrib_multilabel_accumulator_device(device)
-
-
-def _test_distrib_xla_nprocs(index):
-    device = idist.device()
-    _test_distrib_integration_multiclass(device)
-    _test_distrib_integration_multilabel(device)
-    _test_distrib_accumulator_device(device)
-    _test_distrib_multilabel_accumulator_device(device)
-
-
-@pytest.mark.tpu
-@pytest.mark.skipif("NUM_TPU_WORKERS" not in os.environ, reason="Skip if no NUM_TPU_WORKERS in env vars")
-@pytest.mark.skipif(not idist.has_xla_support, reason="Skip if no PyTorch XLA package")
-def test_distrib_xla_nprocs(xmp_executor):
-    n = int(os.environ["NUM_TPU_WORKERS"])
-    xmp_executor(_test_distrib_xla_nprocs, args=(), nprocs=n)
+            if average == "weighted":
+                assert re._weight.device == metric_device, f"{type(re._weight.device)}:{re._weight.device} vs "
+                f"{type(metric_device)}:{metric_device}"
