@@ -10,5 +10,5 @@ elif [ $1 = "fmt" ]; then
 elif [ $1 = "mypy" ]; then
     mypy --config-file mypy.ini
 elif [ $1 = "install" ]; then
-    pip install --upgrade flake8 "black==21.12b0" "usort==1.0.5" "ufmt==2.0.1" "mypy"
+    pip install --upgrade flake8 "black==24.3.0" "usort==1.0.8.post1" "ufmt==2.5.1" "mypy"
 fi
